#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_launch_template
version_added: 1.0.0
version_added_collection: community.aws
short_description: Manage EC2 launch templates
description:
- Create, modify, and delete EC2 Launch Templates, which can be used to
  create individual instances or with Autoscaling Groups.
- The M(amazon.aws.ec2_instance) and M(community.aws.autoscaling_group) modules can, instead of specifying all
  parameters on those tasks, be passed a Launch Template which contains
  settings like instance size, disk type, subnet, and more.
author:
- Ryan Scott Brown (@ryansb)
options:
  template_id:
    description:
    - The ID for the launch template, can be used for all cases except creating a new Launch Template.
    - At least one of O(template_id) and O(template_name) must be specified.
    aliases: [id]
    type: str
  template_name:
    description:
    - The template name. This must be unique in the region-account combination you are using.
    - If no launch template exists with the specified name, a new launch template is created.
    - If a launch template with the specified name already exists and the configuration has not changed,
      nothing happens.
    - If a launch template with the specified name already exists and the configuration has changed,
      a new version of the launch template is created.
    - At least one of O(template_id) and O(template_name) must be specified.
    aliases: [name]
    type: str
  default_version:
    description:
    - Which version should be the default when users spin up new instances based on this template? By default, the latest version will be made the default.
    type: str
    default: latest
  version_description:
    version_added: 5.5.0
    description:
    - The description of a launch template version.
    default: ""
    type: str
  versions_to_delete:
    description:
    - The version numbers of a launch template versions to delete.
    - Use O(default_version) to specify a new default version when deleting the current default version.
    - By default, the latest version will be made the default.
    - Ignored when O(state=present).
    type: list
    elements: int
    version_added: 9.0.0
  state:
    description:
    - Whether the launch template should exist or not.
    - Deleting specific versions of a launch template is not supported at this time.
    choices: [present, absent]
    default: present
    type: str
  block_device_mappings:
    description:
    - The block device mapping. Supplying both a snapshot ID and an encryption
      value as arguments for block-device mapping results in an error. This is
      because only blank volumes can be encrypted on start, and these are not
      created from a snapshot. If a snapshot is the basis for the volume, it
      contains data by definition and its encryption status cannot be changed
      using this action.
    type: list
    elements: dict
    suboptions:
      device_name:
        description: The device name (for example, V(/dev/sdh) or V(xvdh)).
        type: str
      no_device:
        description: Suppresses the specified device included in the block device mapping of the AMI.
        type: str
      virtual_name:
        description: >
          The virtual device name (ephemeralN). Instance store volumes are
          numbered starting from 0. An instance type with 2 available instance
          store volumes can specify mappings for ephemeral0 and ephemeral1. The
          number of available instance store volumes depends on the instance
          type. After you connect to the instance, you must mount the volume.
        type: str
      ebs:
        description: Parameters used to automatically set up EBS volumes when the instance is launched.
        type: dict
        suboptions:
          delete_on_termination:
            description: Indicates whether the EBS volume is deleted on instance termination.
            type: bool
          encrypted:
            description: >
              Indicates whether the EBS volume is encrypted. Encrypted volumes
              can only be attached to instances that support Amazon EBS
              encryption. If you are creating a volume from a snapshot, you
              can't specify an encryption value.
            type: bool
          iops:
            description:
            - The number of I/O operations per second (IOPS) that the volume
              supports. For io1, this represents the number of IOPS that are
              provisioned for the volume. For gp2, this represents the baseline
              performance of the volume and the rate at which the volume
              accumulates I/O credits for bursting. For more information about
              General Purpose SSD baseline performance, I/O credits, and
              bursting, see Amazon EBS Volume Types in the Amazon Elastic
              Compute Cloud User Guide.
            - >
              Condition: This parameter is required for requests to create io1
              volumes; it is not used in requests to create gp2, st1, sc1, or
              standard volumes.
            type: int
          kms_key_id:
            description: The ARN of the AWS Key Management Service (AWS KMS) CMK used for encryption.
            type: str
          snapshot_id:
            description: The ID of the snapshot to create the volume from.
            type: str
          volume_size:
            description:
            - The size of the volume, in GiB.
            - "Default: If you're creating the volume from a snapshot and don't specify a volume size, the default is the snapshot size."
            type: int
          volume_type:
            description: The volume type
            type: str
          throughput:
            description: >
              The throughput to provision for a gp3 volume, with a maximum of 1,000 MiB/s.
              Valid Range - Minimum value of V(125). Maximum value of V(1000).
            type: int
            version_added: 9.0.0
  cpu_options:
    description:
    - Choose CPU settings for the EC2 instances that will be created with this template.
    - For more information, see U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html)
    type: dict
    suboptions:
      core_count:
        description: The number of CPU cores for the instance.
        type: int
      threads_per_core:
        description: >
          The number of threads per CPU core. To disable Intel Hyper-Threading
          Technology for the instance, specify a value of V(1). Otherwise, specify
          the default value of V(2).
        type: int
  credit_specification:
    description: The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.
    type: dict
    suboptions:
      cpu_credits:
        description:
          - The credit option for CPU usage of a T2 or T3 instance. Valid values are C(standard) and C(unlimited).
        type: str
  disable_api_termination:
    description:
      - This helps protect instances from accidental termination.
      - If set to V(true), you can't terminate the instance using the Amazon EC2 console, CLI, or API.
    type: bool
  ebs_optimized:
    description: >
      Indicates whether the instance is optimized for Amazon EBS I/O. This
      optimization provides dedicated throughput to Amazon EBS and an optimized
      configuration stack to provide optimal Amazon EBS I/O performance. This
      optimization isn't available with all instance types. Additional usage
      charges apply when using an EBS-optimized instance.
    type: bool
  elastic_gpu_specifications:
    type: list
    elements: dict
    description: Settings for Elastic GPU attachments. See U(https://aws.amazon.com/ec2/elastic-gpus/) for details.
    suboptions:
      type:
        description: The type of Elastic GPU to attach
        type: str
  iam_instance_profile:
    description: >
      The name or ARN of an IAM instance profile. Requires permissions to
      describe existing instance roles to confirm ARN is properly formed.
    type: str
  image_id:
    description: >
      The AMI ID to use for new instances launched with this template. This
      value is region-dependent since AMIs are not global resources.
    type: str
  instance_initiated_shutdown_behavior:
    description: >
      Indicates whether an instance stops or terminates when you initiate
      shutdown from the instance using the operating system shutdown command.
    choices: [stop, terminate]
    type: str
  instance_market_options:
    description: Options for alternative instance markets, currently only the spot market is supported.
    type: dict
    suboptions:
      market_type:
        description: The market type. This should always be V(spot).
        type: str
      spot_options:
        description: Spot-market specific settings.
        type: dict
        suboptions:
          block_duration_minutes:
            description:
              - The required duration for the Spot Instances (also known as Spot blocks), in minutes.
              - This value must be a multiple of V(60) (V(60), V(120), V(180), V(240), V(300), or V(360)).
            type: int
          instance_interruption_behavior:
            description: The behavior when a Spot Instance is interrupted. The default is V(terminate).
            choices: [hibernate, stop, terminate]
            type: str
          max_price:
            description: The highest hourly price you're willing to pay for this Spot Instance.
            type: str
          spot_instance_type:
            description: The request type to send.
            choices: [one-time, persistent]
            type: str
  instance_type:
    description:
      - The instance type, such as V(c5.2xlarge). For a full list of instance types, see
        U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html).
    type: str
  kernel_id:
    description:
      - The ID of the kernel.
      - We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see
        U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html)
    type: str
  key_name:
    description:
    - The name of the key pair. You can create a key pair using M(amazon.aws.ec2_key).
    - If you do not specify a key pair, you can't connect to the instance
      unless you choose an AMI that is configured to allow users another way to log in.
    type: str
  monitoring:
    description: Settings for instance monitoring.
    type: dict
    suboptions:
      enabled:
        type: bool
        description: Whether to turn on detailed monitoring for new instances. This will incur extra charges.
  network_interfaces:
    description: One or more network interfaces.
    type: list
    elements: dict
    suboptions:
      associate_public_ip_address:
        description: Associates a public IPv4 address with eth0 for a new network interface.
        type: bool
      delete_on_termination:
        description: Indicates whether the network interface is deleted when the instance is terminated.
        type: bool
      description:
        description: A description for the network interface.
        type: str
      device_index:
        description: The device index for the network interface attachment.
        type: int
      groups:
        description: List of security group IDs to include on this instance.
        type: list
        elements: str
      ipv6_address_count:
        description:
          - The number of IPv6 addresses to assign to a network interface.
          - Amazon EC2 automatically selects the IPv6 addresses from the subnet range.
          - You can't use this option if specifying the O(network_interfaces.ipv6_addresses) option.
        type: int
      ipv6_addresses:
        description:
          - A list of one or more specific IPv6 addresses from the IPv6 CIDR block range of your subnet.
          - You can't use this option if you're specifying the O(network_interfaces.ipv6_address_count) option.
        type: list
        elements: str
      network_interface_id:
        description: The eni ID of a network interface to attach.
        type: str
      private_ip_address:
        description: The primary private IPv4 address of the network interface.
        type: str
      subnet_id:
        description: The ID of the subnet for the network interface.
        type: str
  placement:
    description: The placement group settings for the instance.
    type: dict
    suboptions:
      affinity:
        description: The affinity setting for an instance on a Dedicated Host.
        type: str
      availability_zone:
        description: The Availability Zone for the instance.
        type: str
      group_name:
        description: The name of the placement group for the instance.
        type: str
      host_id:
        description: The ID of the Dedicated Host for the instance.
        type: str
      tenancy:
        description: >
          The tenancy of the instance (if the instance is running in a VPC). An
          instance with a tenancy of dedicated runs on single-tenant hardware.
        type: str
  ram_disk_id:
    description: >
      The ID of the RAM disk to launch the instance with. We recommend that you
      use PV-GRUB instead of kernels and RAM disks. For more information, see
      U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html)
    type: str
  security_group_ids:
    description: A list of security group IDs (VPC or EC2-Classic) that the new instances will be added to.
    type: list
    elements: str
  security_groups:
    description: >
      A list of security group names (Default VPC or EC2-Classic) that the new instances will be added to.
      For any VPC other than Default, you must use O(security_group_ids).
    type: list
    elements: str
  source_version:
    description:
      - The version number of the launch template version on which to base the new version.
      - The new version inherits the same launch parameters as the source version, except for parameters that you explicity specify.
      - Snapshots applied to the O(block_device_mappings) are ignored when creating a new version unless they are explicitly included.
    type: str
    default: latest
    version_added: 4.1.0
  tag_specifications:
    description:
    - The tags to apply to the resources when this Launch template is used.
    type: list
    elements: dict
    version_added: 9.0.0
    suboptions:
      resource_type:
        description:
          - The type of resource to tag.
          - If the instance does not include the resource type that you specify, the instance launch fails.
        type: str
        default: instance
        choices:
          - instance
          - volume
          - network-interface
          - spot-instances-request
      tags:
        description:
        - A set of key-value pairs to be applied to the resource type.
        - "Tag key constraints: Tag keys are case-sensitive and accept a maximum of 127 Unicode characters. May not begin with I(aws:)"
        - "Tag value constraints: Tag values are case-sensitive and accept a maximum of 255 Unicode characters."
        type: dict
  user_data:
    description: >
      The Base64-encoded user data to make available to the instance. For more information, see the Linux
      U(http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html) and Windows
      U(http://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data)
      documentation on user-data.
    type: str
  metadata_options:
    description:
    - Configure EC2 Metadata options.
    - For more information see the IMDS documentation
      U(https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-service.html).
    type: dict
    version_added: 1.5.0
    suboptions:
      http_endpoint:
        type: str
        description: This parameter enables or disables the HTTP metadata endpoint on your instances.
        choices: [enabled, disabled]
        default: 'enabled'
      http_put_response_hop_limit:
        type: int
        description:
          - The desired HTTP PUT response hop limit for instance metadata requests.
          - The larger the number, the further instance metadata requests can travel.
        default: 1
      http_tokens:
        type: str
        description: The state of token usage for your instance metadata requests.
        choices: [optional, required]
        default: 'optional'
      http_protocol_ipv6:
        version_added: 3.1.0
        type: str
        description:
          - Whether the instance metadata endpoint is available via IPv6.
        choices: [enabled, disabled]
        default: 'disabled'
      instance_metadata_tags:
        version_added: 3.1.0
        type: str
        description:
          - Whether the instance tags are availble (V(enabled)) via metadata endpoint or not (V(disabled)).
        choices: [enabled, disabled]
        default: 'disabled'
notes:
  - The O(tags) option used has been in release 9.0.0 to be applied to the launch template resource instead of launch template resource.
  - Use O(tag_specifications) to define tags to be applied to resources when this Launch Template is used.
  - Support for O(purge_tags) was added in release 9.0.0.
extends_documentation_fragment:
- amazon.aws.common.modules
- amazon.aws.region.modules
- amazon.aws.boto3
- amazon.aws.tags
"""

EXAMPLES = r"""
- name: Create an ec2 launch template
  amazon.aws.ec2_launch_template:
    name: "my_template"
    image_id: "ami-04b762b4289fba92b"
    key_name: my_ssh_key
    instance_type: t2.micro
    iam_instance_profile: myTestProfile
    disable_api_termination: true

- name: >
    Create a new version of an existing ec2 launch template with a different instance type,
    while leaving an older version as the default version
  amazon.aws.ec2_launch_template:
    name: "my_template"
    default_version: 1
    instance_type: c5.4xlarge

- name: Delete an ec2 launch template
  amazon.aws.ec2_launch_template:
    name: "my_template"
    state: absent

- name: Delete a specific version of an ec2 launch template
  amazon.aws.ec2_launch_template:
    name: "my_template"
    versions_to_delete:
      - 2
    state: absent

- name: Delete a specific version of an ec2 launch template and change the default version
  amazon.aws.ec2_launch_template:
    name: "my_template"
    versions_to_delete:
      - 1
    default_version: 2
    state: absent

- name: Create an ec2 launch template with specific tags
  amazon.aws.ec2_launch_template:
    name: "my_template"
    image_id: "ami-04b762b4289fba92b"
    instance_type: t2.micro
    disable_api_termination: true
    tags:
      Some: tag
      Another: tag

- name: Create an ec2 launch template with different tag for volume and instance
  amazon.aws.ec2_launch_template:
    name: "my_template"
    image_id: "ami-04b762b4289fba92b"
    instance_type: t2.micro
    block_device_mappings:
      - device_name: /dev/sdb
        ebs:
          volume_size: 20
          delete_on_termination: true
          volume_type: standard
    tag_specifications:
      - resource_type: instance
        tags:
          OsType: Linux
      - resource_type: volume
        tags:
          foo: bar
"""

RETURN = r"""
latest_version:
  description: The latest available version number of the launch template.
  returned: when RV(latest_template) has a version number.
  type: int
default_version:
  description: The version that will be used if only the template name is specified. Often this is the same as the latest version, but not always.
  returned: when RV(default_template) has a version number.
  type: int
template:
  description: Latest available version of the launch template.
  returned: when O(state=present)
  type: complex
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time launch template was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the launch template.
      type: str
      returned: always
    default_version_number:
      description: The version number of the default version of the launch template.
      type: int
      returned: always
    latest_version_number:
      description: The version number of the latest version of the launch template.
      type: int
      returned: always
    tags:
      description: A dictionary of tags assigned to image.
      returned: when AMI is created or already exists
      type: dict
      sample: {
          "Env": "devel",
          "Name": "nat-server"
      }
versions:
  description: All available versions of the launch template.
  returned: when O(state=present)
  type: list
  elements: dict
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time the version was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the version.
      type: str
      returned: always
    default_version:
      description: Indicates whether the version is the default version.
      type: bool
      returned: always
    version_number:
      description: The version number.
      type: int
      returned: always
    version_description:
      description: The description for the version.
      type: str
      returned: always
    launch_template_data:
      description: Information about the launch template.
      returned: always
      type: dict
      contains:
        kernel_id:
          description:
            - The ID of the kernel.
          returned: if applicable
          type: str
        image_id:
          description: The ID of the AMI or a Systems Manager parameter.
          type: str
          returned: if applicable
        instance_type:
          description: The instance type.
          type: str
          returned: if applicable
        key_name:
          description: The name of the key pair.
          type: str
          returned: if applicable
        monitoring:
          description: The monitoring for the instance.
          type: dict
          returned: if applicable
          contains:
            enabled:
              description: Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.
              type: bool
              returned: always
        placement:
          description: The placement of the instance.
          type: dict
          returned: if applicable
          contains:
            availability_zone:
              description: The Availability Zone of the instance.
              type: str
              returned: if applicable
            affinity:
              description: The affinity setting for the instance on the Dedicated Host.
              type: str
              returned: if applicable
            group_name:
              description: The name of the placement group for the instance.
              type: str
              returned: if applicable
            host_id:
              description: The ID of the Dedicated Host for the instance.
              type: str
              returned: if applicable
            tenancy:
              description: The tenancy of the instance.
              type: str
              returned: if applicable
            host_resource_group_arn:
              description: The ARN of the host resource group in which to launch the instances.
              type: str
              returned: if applicable
            partition_number:
              description: The number of the partition the instance should launch in.
              type: int
              returned: if applicable
            group_id:
              description: The Group ID of the placement group.
              type: str
              returned: if applicable
        ebs_optimized:
          description:
            - Indicates whether the instance is optimized for Amazon EBS I/O.
          type: bool
          returned: always
        iam_instance_profile:
          description:
            - The IAM instance profile.
          type: dict
          returned: if application
          contains:
            arn:
              description: The Amazon Resource Name (ARN) of the instance profile.
              type: str
              returned: always
            name:
              description: The name of the instance profile.
              type: str
              returned: always
        block_device_mappings:
          description: The block device mappings.
          type: list
          elements: dict
          returned: if applicable
          contains:
            device_name:
              description: The device name.
              type: str
              returned: always
            virtual_name:
              description: The virtual device name.
              type: str
              returned: always
            ebs:
              description: Information about the block device for an EBS volume.
              type: str
              returned: if applicable
              contains:
                encrypted:
                  description: Indicates whether the EBS volume is encrypted.
                  type: bool
                  returned: always
                delete_on_termination:
                  description: Indicates whether the EBS volume is deleted on instance termination.
                  type: bool
                  returned: always
                iops:
                  description: The number of I/O operations per second (IOPS) that the volume supports.
                  type: int
                  returned: always
                kms_key_id:
                  description: The ARN of the Key Management Service (KMS) CMK used for encryption.
                  type: int
                  returned: always
                snapshot_id:
                  description: The ID of the snapshot.
                  type: str
                  returned: always
                volume_size:
                  description: The size of the volume, in GiB.
                  type: int
                  returned: always
                volume_type:
                  description: The volume type.
                  type: str
                  returned: always
                throughput:
                  description: The throughput that the volume supports, in MiB/s.
                  type: int
                  returned: always
            no_device:
              description: To omit the device from the block device mapping, specify an empty string.
              type: str
        network_interfaces:
          description: The network interfaces.
          type: list
          elements: dict
          returned: if applicable
          contains:
            associate_carrier_ip_address:
              description: Indicates whether to associate a Carrier IP address with eth0 for a new network interface.
              type: bool
              returned: always
            associate_public_ip_address:
              description: Indicates whether to associate a public IPv4 address with eth0 for a new network interface.
              type: bool
              returned: always
            delete_on_termination:
              description: Indicates whether the network interface is deleted when the instance is terminated.
              type: bool
              returned: always
            description:
              description: A description for the network interface.
              type: str
              returned: always
            device_index:
              description: The device index for the network interface attachment.
              type: int
              returned: always
            groups:
              description: The IDs of one or more security groups.
              type: list
              elements: str
              returned: if applicable
            interface_type:
              description: The type of network interface.
              type: str
              returned: always
            ipv6_address_count:
              description: The number of IPv6 addresses for the network interface.
              type: int
              returned: if applicable
            ipv6_addresses:
              description: The IPv6 addresses for the network interface.
              returned: if applicable
              type: list
              elements: dict
              contains:
                ipv6_address:
                  description: The IPv6 address.
                  type: str
                  returned: always
                is_primary_ipv6:
                  description: Determines if an IPv6 address associated with a network interface is the primary IPv6 address.
                  type: bool
                  returned: always
            network_interface_id:
              description: The ID of the network interface.
              type: str
              returned: always
            private_ip_address:
              description: The primary private IPv4 address of the network interface.
              type: str
              returned: if applicable
            private_ip_addresses:
              description: A list of private IPv4 addresses.
              type: list
              elements: str
              returned: if applicable
              contains:
                primary:
                  description: Indicates whether the private IPv4 address is the primary private IPv4 address.
                  type: bool
                  returned: always
                private_ip_address:
                  description: The private IPv4 address.
                  type: bool
                  returned: always
            secondary_private_ip_address_count:
              description: The number of secondary private IPv4 addresses for the network interface.
              type: int
              returned: if applicable
            subnet_id:
              description: The ID of the subnet for the network interface.
              type: str
              returned: always
            network_card_index:
              description: The index of the network card.
              type: int
              returned: if applicable
            ipv4_prefixes:
              description: A list of IPv4 prefixes assigned to the network interface.
              type: list
              elements: dict
              returned: if applicable
              contains:
                ipv4_prefix:
                  description: The IPv4 delegated prefixes assigned to the network interface.
                  type: str
                  returned: always
            ipv4_prefix_count:
              description: The number of IPv4 prefixes that Amazon Web Services automatically assigned to the network interface.
              type: int
              returned: if applicable
            ipv6_prefixes:
              description: A list of IPv6 prefixes assigned to the network interface.
              type: list
              elements: dict
              returned: if applicable
              contains:
                ipv6_prefix:
                  description: The IPv6 delegated prefixes assigned to the network interface.
                  type: str
                  returned: always
            ipv6_prefix_count:
              description: The number of IPv6 prefixes that Amazon Web Services automatically assigned to the network interface.
              type: int
              returned: if applicable
            primary_ipv6:
              description: The primary IPv6 address of the network interface.
              type: str
              returned: if applicable
            ena_srd_specification:
              description: Contains the ENA Express settings for instances launched from your launch template.
              type: dict
              returned: if applicable
              contains:
                ena_srd_enabled:
                  description: Indicates whether ENA Express is enabled for the network interface.
                  type: bool
                  returned: always
                ena_srd_udp_specification:
                  description: Configures ENA Express for UDP network traffic.
                  type: dict
                  returned: always
                  contains:
                    ena_srd_udp_enabled:
                      description: Indicates whether UDP traffic to and from the instance uses ENA Express.
                      type: bool
                      returned: always
            connection_tracking_specification:
              description:
                - A security group connection tracking specification that enables you to set the timeout
                  for connection tracking on an Elastic network interface.
              type: dict
              returned: if applicable
              contains:
                tcp_established_timeout:
                  description: Timeout (in seconds) for idle TCP connections in an established state.
                  type: int
                  returned: always
                udp_timeout:
                  description:
                    - Timeout (in seconds) for idle UDP flows that have seen traffic only in a single direction
                      or a single request-response transaction.
                  type: int
                  returned: always
                udp_stream_timeout:
                  description:
                    - Timeout (in seconds) for idle UDP flows classified as streams which have seen more
                      than one request-response transaction.
                  type: int
                  returned: always
        ram_disk_id:
          description: The ID of the RAM disk, if applicable.
          type: str
          returned: if applicable
        disable_api_termination:
          description: If set to true, indicates that the instance cannot be terminated using the Amazon EC2 console, command line tool, or API.
          type: bool
          returned: if applicable
        instance_initiated_shutdown_behavior:
          description: Indicates whether an instance stops or terminates when you initiate shutdown from the instance.
          type: str
          returned: if applicable
        user_data:
          description: The user data for the instance.
          type: str
          returned: if applicable
        tag_specifications:
          description: The tags that are applied to the resources that are created during instance launch.
          type: list
          elements: dict
          returned: if applicable
          contains:
            resource_type:
              description: The type of resource to tag.
              type: str
              returned: always
            tags:
              description: The tags for the resource.
              type: list
              elements: dict
              contains:
                key:
                  description: The key of the tag.
                  type: str
                  returned: always
                value:
                  description: The value of the tag.
                  type: str
                  returned: always
        enclave_options:
          description: Indicates whether the instance is enabled for Amazon Web Services Nitro Enclaves.
          type: dict
          returned: if applicable
          contains:
            enabled:
              description: If this parameter is set to true, the instance is enabled for Amazon Web Services Nitro Enclaves.
              type: bool
              returned: always
        metadata_options:
          description: The metadata options for the instance.
          type: dict
          returned: if applicable
          contains:
            state:
              description: The state of the metadata option changes.
              type: str
              returned: if applicable
            http_tokens:
              description: Indicates whether IMDSv2 is required.
              type: str
              returned: if applicable
            http_put_response_hop_limit:
              description: The desired HTTP PUT response hop limit for instance metadata requests.
              type: int
              returned: if applicable
            http_endpoint:
              description: Enables or disables the HTTP metadata endpoint on your instances.
              type: str
              returned: if applicable
            http_protocol_ipv6:
              description: Enables or disables the IPv6 endpoint for the instance metadata service.
              type: str
              returned: if applicable
            instance_metadata_tags:
              description: Set to enabled to allow access to instance tags from the instance metadata.
              type: str
              returned: if applicable
        cpu_options:
          description: The CPU options for the instance.
          type: dict
          returned: if applicable
          contains:
            core_count:
              description: The number of CPU cores for the instance.
              type: int
              returned: if applicable
            threads_per_core:
              description: The number of threads per CPU core.
              type: int
              returned: if applicable
            amd_sev_snp:
              description: Indicates whether the instance is enabled for AMD SEV-SNP.
              type: int
              returned: if applicable
        security_group_ids:
          description: The security group IDs.
          type: list
          elements: str
          returned: if applicable
        security_groups:
          description: The security group names.
          type: list
          elements: str
          returned: if applicable
      sample: {
          "block_device_mappings": [
              {
                  "device_name": "/dev/sdb",
                  "ebs": {
                      "delete_on_termination": true,
                      "encrypted": true,
                      "volumeSize": 5
                  }
              }
          ],
          "ebs_optimized": false,
          "image_id": "ami-0231217be14a6f3ba",
          "instance_type": "t2.micro",
          "network_interfaces": [
              {
                  "associate_public_ip_address": false,
                  "device_index": 0,
                  "ipv6_addresses": [
                      {
                          "ipv6_address": "2001:0:130F:0:0:9C0:876A:130B"
                      }
                  ]
              }
          ]
      }
latest_template:
  description: The latest available version of the launch template.
  returned: when O(state=present)
  type: complex
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time launch template was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the launch template.
      type: str
      returned: always
    default_version_number:
      description: The version number of the default version of the launch template.
      type: int
      returned: always
    latest_version_number:
      description: The version number of the latest version of the launch template.
      type: int
      returned: always
    tags:
      description: A dictionary of tags assigned to image.
      returned: when AMI is created or already exists
      type: dict
      sample: {
          "Env": "devel",
          "Name": "nat-server"
      }
default_template:
  description:
    - The launch template version that will be used if only the template name is specified.
    - Often this is the same as the latest version, but not always.
  returned: when O(state=present)
  type: complex
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time launch template was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the launch template.
      type: str
      returned: always
    default_version_number:
      description: The version number of the default version of the launch template.
      type: int
      returned: always
    latest_version_number:
      description: The version number of the latest version of the launch template.
      type: int
      returned: always
    tags:
      description: A dictionary of tags assigned to image.
      returned: when AMI is created or already exists
      type: dict
      sample: {
          "Env": "devel",
          "Name": "nat-server"
      }
deleted_template:
  description: information about a launch template deleted.
  returned: when O(state=absent)
  type: complex
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    create_time:
      description: The time launch template was created.
      type: str
      returned: always
    created_by:
      description: The principal that created the launch template.
      type: str
      returned: always
    default_version_number:
      description: The version number of the default version of the launch template.
      type: int
      returned: always
    latest_version_number:
      description: The version number of the latest version of the launch template.
      type: int
      returned: always
    tags:
      description: A dictionary of tags assigned to image.
      returned: when AMI is created or already exists
      type: dict
      sample: {
          "Env": "devel",
          "Name": "nat-server"
      }
deleted_versions:
  description: Information about deleted launch template versions.
  returned: when O(state=absent)
  type: list
  elements: dict
  contains:
    launch_template_id:
      description: The ID of the launch template.
      type: str
      returned: always
    launch_template_name:
      description: The name of the launch template.
      type: str
      returned: always
    version_number:
      description: The version number of the launch template.
      type: int
      returned: always
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from uuid import uuid4

from ansible.module_utils._text import to_text
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import normalize_boto3_result
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_launch_template
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_launch_template_version
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_launch_template
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_launch_template_versions
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_launch_template_versions
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_launch_templates
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import determine_iam_arn_from_name
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import modify_launch_template
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import scrub_none_parameters


def find_existing(client, module: AnsibleAWSModule) -> Tuple[Optional[Dict[str, Any]], Optional[List[Dict[str, Any]]]]:
    launch_template = None
    launch_template_versions = []
    params = {}
    template_id = module.params.get("template_id")
    template_name = module.params.get("template_name")
    if template_id:
        params["launch_template_ids"] = [template_id]
    else:
        params["launch_template_names"] = [template_name]
    launch_templates = describe_launch_templates(client, **params)
    if launch_templates:
        launch_template = launch_templates[0]
        launch_template_versions = describe_launch_template_versions(
            client, LaunchTemplateId=launch_template["LaunchTemplateId"]
        )
    return normalize_boto3_result(launch_template), normalize_boto3_result(launch_template_versions)


def params_to_launch_data(
    template_params: Dict[str, Any], iam_instance_profile_arn: Optional[str] = None
) -> Dict[str, Any]:
    if iam_instance_profile_arn:
        template_params["iam_instance_profile"] = {"arn": iam_instance_profile_arn}
    for interface in template_params.get("network_interfaces") or []:
        if interface.get("ipv6_addresses"):
            interface["ipv6_addresses"] = [{"ipv6_address": x} for x in interface["ipv6_addresses"]]
    params = snake_dict_to_camel_dict(
        dict((k, v) for k, v in template_params.items() if v is not None),
        capitalize_first=True,
    )
    return params


def validate_string_as_int(module: AnsibleAWSModule, version: str, param_name: str) -> int:
    try:
        return int(version)
    except ValueError:
        module.fail_json(msg=f'{param_name} param was not a valid integer, got "{version}"')


def validate_version_deletion(
    module: AnsibleAWSModule, launch_template_id: str, existing_versions: List[Dict[str, Any]]
) -> Tuple[List[str], Optional[int]]:
    versions_to_delete = module.params.get("versions_to_delete")
    launch_template_versions_to_delete = []
    default_version_to_set = None
    if versions_to_delete:
        unique_versions_to_delete = list(set(versions_to_delete))
        launch_template_versions_to_delete = [
            t["VersionNumber"] for t in existing_versions if t["VersionNumber"] in unique_versions_to_delete
        ]
        if len(launch_template_versions_to_delete) != len(unique_versions_to_delete):
            missing = [m for m in unique_versions_to_delete if m not in launch_template_versions_to_delete]
            module.fail_json(
                msg=f"The following versions {missing} do not exist for launch template id '{launch_template_id}'."
            )

        remaining_versions = [
            t["VersionNumber"]
            for t in existing_versions
            if t["VersionNumber"] not in launch_template_versions_to_delete
        ]

        # Find the default version
        default_version = module.params.get("default_version")
        if default_version in (None, ""):
            default_version_int = [t["VersionNumber"] for t in existing_versions if t["DefaultVersion"]][0]
        elif default_version == "latest":
            default_version_int = max(remaining_versions, default=None)
            default_version_to_set = default_version_int
        else:
            default_version_int = validate_string_as_int(module, default_version, "default_version")
            default_version_to_set = default_version_int

        # Ensure we are not deleting the default version
        if default_version_int in launch_template_versions_to_delete or not remaining_versions:
            module.fail_json(msg="Cannot delete the launch template default version.")

        if default_version_to_set and default_version_to_set not in remaining_versions:
            module.fail_json(
                msg=f"Could not set version '{default_version_to_set}' as default, "
                "the launch template version was not found for the specified launch template id '{launch_template_id}'."
            )
    else:
        # By default delete all non default version before the launch template deletion
        launch_template_versions_to_delete = [t["VersionNumber"] for t in existing_versions if not t["DefaultVersion"]]

    return [to_text(v) for v in launch_template_versions_to_delete], default_version_to_set


def ensure_absent(
    client, module: AnsibleAWSModule, existing: Optional[Dict[str, Any]], existing_versions: List[Dict[str, Any]]
) -> None:
    deleted_versions = []
    deleted_template = {}
    changed = False

    if existing:
        launch_template_id = existing["LaunchTemplateId"]
        v_to_delete, v_default = validate_version_deletion(module, launch_template_id, existing_versions)

        # Update default version
        if v_default:
            changed = True
            if not module.check_mode:
                modify_launch_template(
                    client,
                    LaunchTemplateId=launch_template_id,
                    ClientToken=uuid4().hex,
                    DefaultVersion=to_text(v_default),
                )
        # Delete versions
        if v_to_delete:
            changed = True
            if not module.check_mode:
                response = delete_launch_template_versions(
                    client, launch_template_id=launch_template_id, versions=v_to_delete
                )
                if response["UnsuccessfullyDeletedLaunchTemplateVersions"]:
                    module.warn(
                        f"Failed to delete template versions {response['UnsuccessfullyDeletedLaunchTemplateVersions']} on"
                        f" launch template {launch_template_id}"
                    )
                deleted_versions = [
                    camel_dict_to_snake_dict(v) for v in response["SuccessfullyDeletedLaunchTemplateVersions"]
                ]

        # Delete the launch template when a list of versions was not specified
        if not module.params.get("versions_to_delete"):
            changed = True
            if not module.check_mode:
                deleted_template = delete_launch_template(client, launch_template_id=launch_template_id)
                deleted_template = camel_dict_to_snake_dict(deleted_template, ignore_list=["Tags"])
                if "tags" in deleted_template:
                    deleted_template["tags"] = boto3_tag_list_to_ansible_dict(deleted_template.get("tags"))

    module.exit_json(changed=changed, deleted_versions=deleted_versions, deleted_template=deleted_template)


def add_launch_template_version(
    client,
    module: AnsibleAWSModule,
    launch_template_id: str,
    launch_template_data: Dict[str, Any],
    existing_versions: List[Dict[str, Any]],
    most_recent_version_number: str,
) -> int:
    source_version = module.params.get("source_version")
    version_description = module.params.get("version_description")

    params = {
        "LaunchTemplateId": launch_template_id,
        "ClientToken": uuid4().hex,
        "VersionDescription": version_description,
    }

    if source_version == "latest":
        params.update({"SourceVersion": most_recent_version_number})
    elif source_version not in (None, ""):
        # Source version passed as int
        source_version_int = validate_string_as_int(module, source_version, "source_version")
        # get source template version
        next_source_version = next(
            (v for v in existing_versions if v["VersionNumber"] == source_version_int),
            None,
        )
        if next_source_version is None:
            module.fail_json(msg=f'source_version does not exist, got "{source_version}"')
        params.update({"SourceVersion": str(next_source_version["VersionNumber"])})

    if module.check_mode:
        module.exit_json(changed=True, msg="Would have created launch template version if not in check mode.")

    # Create Launch template version
    launch_template_version = create_launch_template_version(
        client, launch_template_data=launch_template_data, **params
    )
    return launch_template_version["VersionNumber"]


def ensure_default_version(
    client,
    module: AnsibleAWSModule,
    launch_template_id: str,
    current_default_version_number: int,
    most_recent_version_number: int,
) -> bool:
    # Modify default version
    default_version = module.params.get("default_version")
    changed = False
    if default_version not in (None, ""):
        if default_version == "latest":
            default_version = to_text(most_recent_version_number)
        else:
            default_version = to_text(validate_string_as_int(module, default_version, "default_version"))
        if to_text(current_default_version_number) != default_version:
            changed = True
            if not module.check_mode:
                modify_launch_template(
                    client,
                    LaunchTemplateId=launch_template_id,
                    ClientToken=uuid4().hex,
                    DefaultVersion=default_version,
                )
    return changed


def format_module_output(client, module: AnsibleAWSModule) -> Dict[str, Any]:
    # Describe launch template
    template, template_versions = find_existing(client, module)
    template = camel_dict_to_snake_dict(template, ignore_list=["Tags"])
    if "tags" in template:
        template["tags"] = boto3_tag_list_to_ansible_dict(template.get("tags"))
    template_versions = [camel_dict_to_snake_dict(v) for v in template_versions]
    result = {
        "template": template,
        "versions": template_versions,
        "default_template": [v for v in template_versions if v.get("default_version")][0],
        "latest_template": [
            v
            for v in template_versions
            if (v.get("version_number") and int(v["version_number"]) == int(template["latest_version_number"]))
        ][0],
    }
    if "version_number" in result["default_template"]:
        result["default_version"] = result["default_template"]["version_number"]
    if "version_number" in result["latest_template"]:
        result["latest_version"] = result["latest_template"]["version_number"]
    return result


def ensure_present(
    client,
    module: AnsibleAWSModule,
    template_options: Dict[str, Any],
    existing: Optional[Dict[str, Any]],
    existing_versions: List[Dict[str, Any]],
) -> None:
    template_name = module.params["template_name"]
    tags = module.params["tags"]
    tag_specifications = module.params.get("tag_specifications")
    version_description = module.params.get("version_description")
    iam_instance_profile = module.params.get("iam_instance_profile")
    # IAM instance profile
    if iam_instance_profile:
        iam_instance_profile = determine_iam_arn_from_name(module.client("iam"), iam_instance_profile)
    # Convert Launch template data
    launch_template_data = params_to_launch_data(
        dict((k, v) for k, v in module.params.items() if k in template_options), iam_instance_profile
    )
    # Tag specifications
    if tag_specifications:
        boto3_tag_specs = []
        for tag_spec in tag_specifications:
            boto3_tag_specs.extend(boto3_tag_specifications(tag_spec["tags"], types=tag_spec["resource_type"]))
        launch_template_data["TagSpecifications"] = boto3_tag_specs
    launch_template_data = scrub_none_parameters(launch_template_data, descend_into_lists=True)
    changed = False

    if not (existing or existing_versions):
        # Create Launch template
        if module.check_mode:
            module.exit_json(changed=True, msg="Would have created launch template if not in check mode.")
        create_launch_template(
            client,
            launch_template_name=template_name,
            launch_template_data=launch_template_data,
            tags=tags,
            ClientToken=uuid4().hex,
            VersionDescription=version_description,
        )
        changed = True
    else:
        launch_template_id = existing["LaunchTemplateId"]
        default_version_number = existing["DefaultVersionNumber"]
        most_recent = sorted(existing_versions, key=lambda x: x["VersionNumber"])[-1]
        most_recent_version_number = most_recent["VersionNumber"]
        if not (
            launch_template_data == most_recent["LaunchTemplateData"]
            and version_description == most_recent.get("VersionDescription", "")
        ):
            changed = True
            most_recent_version_number = add_launch_template_version(
                client,
                module,
                launch_template_id,
                launch_template_data,
                existing_versions,
                str(most_recent["VersionNumber"]),
            )

        # Ensure default version
        changed |= ensure_default_version(
            client, module, launch_template_id, default_version_number, most_recent_version_number
        )
        # Ensure tags
        changed |= ensure_ec2_tags(
            client,
            module,
            launch_template_id,
            resource_type="launch-template",
            tags=tags,
            purge_tags=module.params["purge_tags"],
        )

    module.exit_json(changed=changed, **format_module_output(client, module))


def main():
    template_options = dict(
        block_device_mappings=dict(
            type="list",
            elements="dict",
            options=dict(
                device_name=dict(),
                ebs=dict(
                    type="dict",
                    options=dict(
                        delete_on_termination=dict(type="bool"),
                        encrypted=dict(type="bool"),
                        iops=dict(type="int"),
                        kms_key_id=dict(),
                        snapshot_id=dict(),
                        volume_size=dict(type="int"),
                        volume_type=dict(),
                        throughput=dict(type="int"),
                    ),
                ),
                no_device=dict(),
                virtual_name=dict(),
            ),
        ),
        cpu_options=dict(
            type="dict",
            options=dict(
                core_count=dict(type="int"),
                threads_per_core=dict(type="int"),
            ),
        ),
        credit_specification=dict(
            dict(type="dict"),
            options=dict(
                cpu_credits=dict(),
            ),
        ),
        disable_api_termination=dict(type="bool"),
        ebs_optimized=dict(type="bool"),
        elastic_gpu_specifications=dict(
            options=dict(type=dict()),
            type="list",
            elements="dict",
        ),
        image_id=dict(),
        instance_initiated_shutdown_behavior=dict(choices=["stop", "terminate"]),
        instance_market_options=dict(
            type="dict",
            options=dict(
                market_type=dict(),
                spot_options=dict(
                    type="dict",
                    options=dict(
                        block_duration_minutes=dict(type="int"),
                        instance_interruption_behavior=dict(choices=["hibernate", "stop", "terminate"]),
                        max_price=dict(),
                        spot_instance_type=dict(choices=["one-time", "persistent"]),
                    ),
                ),
            ),
        ),
        instance_type=dict(),
        kernel_id=dict(),
        key_name=dict(),
        monitoring=dict(
            type="dict",
            options=dict(enabled=dict(type="bool")),
        ),
        metadata_options=dict(
            type="dict",
            options=dict(
                http_endpoint=dict(choices=["enabled", "disabled"], default="enabled"),
                http_put_response_hop_limit=dict(type="int", default=1),
                http_tokens=dict(choices=["optional", "required"], default="optional"),
                http_protocol_ipv6=dict(choices=["disabled", "enabled"], default="disabled"),
                instance_metadata_tags=dict(choices=["disabled", "enabled"], default="disabled"),
            ),
        ),
        network_interfaces=dict(
            type="list",
            elements="dict",
            options=dict(
                associate_public_ip_address=dict(type="bool"),
                delete_on_termination=dict(type="bool"),
                description=dict(),
                device_index=dict(type="int"),
                groups=dict(type="list", elements="str"),
                ipv6_address_count=dict(type="int"),
                ipv6_addresses=dict(type="list", elements="str"),
                network_interface_id=dict(),
                private_ip_address=dict(),
                subnet_id=dict(),
            ),
        ),
        placement=dict(
            options=dict(
                affinity=dict(),
                availability_zone=dict(),
                group_name=dict(),
                host_id=dict(),
                tenancy=dict(),
            ),
            type="dict",
        ),
        ram_disk_id=dict(),
        security_group_ids=dict(type="list", elements="str"),
        security_groups=dict(type="list", elements="str"),
        user_data=dict(),
    )

    argument_spec = dict(
        state=dict(choices=["present", "absent"], default="present"),
        template_name=dict(aliases=["name"]),
        template_id=dict(aliases=["id"]),
        default_version=dict(default="latest"),
        source_version=dict(default="latest"),
        version_description=dict(default=""),
        iam_instance_profile=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        versions_to_delete=dict(type="list", elements="int"),
        tag_specifications=dict(
            type="list",
            elements="dict",
            options=dict(
                resource_type=dict(
                    type="str",
                    default="instance",
                    choices=["instance", "volume", "network-interface", "spot-instances-request"],
                ),
                tags=dict(type="dict"),
            ),
        ),
    )

    argument_spec.update(template_options)

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=[
            ("template_name", "template_id"),
        ],
        supports_check_mode=True,
    )

    state = module.params.get("state")
    client = module.client("ec2")
    launch_template, launch_template_versions = find_existing(client, module)

    try:
        if state == "present":
            ensure_present(client, module, template_options, launch_template, launch_template_versions)
        else:
            ensure_absent(client, module, launch_template, launch_template_versions)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_image_updates_count_v1_info
short_description: Information module for Network Device Image Updates Count V1
description:
- Get all Network Device Image Updates Count V1.
- Returns the count of network device image updates based on the given filter criteria.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id query parameter. Update id which is unique for each network device under the parentId.
    type: str
  parentId:
    description:
    - ParentId query parameter. Updates that have this parent id.
    type: str
  networkDeviceId:
    description:
    - NetworkDeviceId query parameter. Network device id.
    type: str
  status:
    description:
    - Status query parameter. Status of the image update. Available values FAILURE, SUCCESS, IN_PROGRESS, PENDING.
    type: str
  imageName:
    description:
    - ImageName query parameter. Software image name for the update.
    type: str
  hostName:
    description:
    - >
      HostName query parameter. Host name of the network device for the image update. Supports case-insensitive
      partial search.
    type: str
  managementAddress:
    description:
    - ManagementAddress query parameter. Management address of the network device.
    type: str
  startTime:
    description:
    - StartTime query parameter. Image update started after the given time (as milliseconds since UNIX epoch).
    type: float
  endTime:
    description:
    - EndTime query parameter. Image update started before the given time (as milliseconds since UNIX epoch).
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) CountOfNetworkDeviceImageUpdatesV1
  description: Complete reference of the CountOfNetworkDeviceImageUpdatesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!count-of-network-device-image-updates
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.count_of_network_device_image_updates_v1,

  - Paths used are
    get /dna/intent/api/v1/networkDeviceImageUpdates/count,

"""

EXAMPLES = r"""
- name: Get all Network Device Image Updates Count V1
  cisco.dnac.network_device_image_updates_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    parentId: string
    networkDeviceId: string
    status: string
    imageName: string
    hostName: string
    managementAddress: string
    startTime: 0
    endTime: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: eox_status_device_v1_info
short_description: Information module for Eox Status Device V1
description:
- Get all Eox Status Device V1.
- Get Eox Status Device V1 by id.
- Retrieves EoX details for a device.
- Retrieves EoX status for all devices in the network.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceId:
    description:
    - DeviceId path parameter. Device instance UUID.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for EoX GetEoXDetailsPerDeviceV1
  description: Complete reference of the GetEoXDetailsPerDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-eo-x-details-per-device
- name: Cisco DNA Center documentation for EoX GetEoXStatusForAllDevicesV1
  description: Complete reference of the GetEoXStatusForAllDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-eo-x-status-for-all-devices
notes:
  - SDK Method used are
    eox.Eox.get_eox_details_per_device_v1,
    eox.Eox.get_eox_status_for_all_devices_v1,

  - Paths used are
    get /dna/intent/api/v1/eox-status/device,
    get /dna/intent/api/v1/eox-status/device/{deviceId},

"""

EXAMPLES = r"""
- name: Get all Eox Status Device V1
  cisco.dnac.eox_status_device_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

- name: Get Eox Status Device V1 by id
  cisco.dnac.eox_status_device_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "deviceId": "string",
        "alertCount": 0,
        "eoxDetails": [
          {
            "name": "string",
            "bulletinHeadline": "string",
            "bulletinName": "string",
            "bulletinNumber": "string",
            "bulletinURL": "string",
            "endOfHardwareNewServiceAttachmentDate": "string",
            "endOfHardwareServiceContractRenewalDate": "string",
            "endOfLastHardwareShipDate": "string",
            "endOfLifeExternalAnnouncementDate": "string",
            "endOfSignatureReleasesDate": "string",
            "endOfSoftwareVulnerabilityOrSecuritySupportDate": "string",
            "endOfSoftwareVulnerabilityOrSecuritySupportDateHw": "string",
            "endOfSaleDate": "string",
            "endOfLifeDate": "string",
            "lastDateOfSupport": "string",
            "endOfSoftwareMaintenanceReleasesDate": "string",
            "eoxAlertType": "string",
            "eoXPhysicalType": "string",
            "bulletinPID": "string"
          }
        ],
        "scanStatus": "string",
        "comments": [
          "string"
        ],
        "lastScanTime": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: system_issue_definitions_count_v1_info
short_description: Information module for System Issue Definitions Count V1
description:
- Get all System Issue Definitions Count V1.
- >
   Get the count of system defined issue definitions based on provided filters. Supported filters are id, name,
   profileId and definition enable status. For detailed information about the usage of the API, please refer to the
   Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-issueAndHealthDefinitions-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceType:
    description:
    - >
      DeviceType query parameter. These are the device families/types supported for system issue definitions. If
      no input is made on device type, all device types are considered.
    type: str
  profileId:
    description:
    - >
      ProfileId query parameter. The profile identier to fetch the profile associated issue defintions. The
      default is `global`. Please refer Network design profiles documentation for more details.
    type: str
  id:
    description:
    - >
      Id query parameter. The definition identifier. Examples id=015d9cba-4f53-4087-8317-7e49e5ffef46 (single
      entity id request) id=015d9cba-4f53-4087-8317-7e49e5ffef46&id=015d9cba-4f53-4087-8317-7e49e5ffef47 (multiple
      ids in the query param).
    type: str
  name:
    description:
    - >
      Name query parameter. The list of system defined issue names. (Ex."BGP_Down") Examples name=BGP_Down (single
      entity uuid requested) name=BGP_Down&name=BGP_Flap (multiple issue names separated by & operator).
    type: str
  priority:
    description:
    - >
      Priority query parameter. Issue priority, possible values are P1, P2, P3, P4. `P1` A critical issue that
      needs immediate attention and can have a wide impact on network operations. `P2` A major issue that can
      potentially impact multiple devices or clients. `P3` A minor issue that has a localized or minimal impact.
      `P4` A warning issue that may not be an immediate problem but addressing it can optimize the network
      performance.
    type: str
  issueEnabled:
    description:
    - IssueEnabled query parameter. The enablement status of the issue definition, either true or false.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTheCountOfSystemDefinedIssueDefinitionsBasedOnProvidedFiltersV1
  description: Complete reference of the GetTheCountOfSystemDefinedIssueDefinitionsBasedOnProvidedFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-count-of-system-defined-issue-definitions-based-on-provided-filters
notes:
  - SDK Method used are
    issues.Issues.get_the_count_of_system_defined_issue_definitions_based_on_provided_filters_v1,

  - Paths used are
    get /dna/intent/api/v1/systemIssueDefinitions/count,

"""

EXAMPLES = r"""
- name: Get all System Issue Definitions Count V1
  cisco.dnac.system_issue_definitions_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceType: string
    profileId: string
    id: string
    name: string
    priority: string
    issueEnabled: True
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for cp_mgmt_hosts
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: cp_mgmt_hosts
short_description: Manages HOSTS resource module
description:
  - This resource module allows for addition, deletion, or modification of CP MGMT Hosts.
  - This resource module also takes care of gathering Hosts config facts
version_added: "5.0.0"
author: Ansible Security Automation Team (@justjais) <https://github.com/ansible-security>-
options:
  config:
    description: A dictionary of HOSTS options
    type: dict
    suboptions:
      name:
        description: Object name. Must be unique in the domain.
        type: str
      ip_address:
        description: IPv4 or IPv6 address. If both addresses are required use ipv4-address
          and ipv6-address fields explicitly.
        type: str
      ipv4_address:
        description: IPv4 address.
        type: str
      ipv6_address:
        description: IPv4 address.
        type: str
      interfaces:
        description: Host interfaces.
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - Interface name.
            type: str
          subnet:
            description:
              - IPv4 or IPv6 network address.
              - If both addresses are required use subnet4 and subnet6 fields explicitly.
            type: str
          subnet4:
            description:
              - IPv4 network address.
            type: str
          subnet6:
            description:
              - IPv6 network address.
            type: str
          mask_length:
            description:
              - IPv4 or IPv6 network mask length. If both masks are required use mask-length4 and
                mask-length6 fields explicitly.
              - Instead of IPv4 mask length it is possible to specify IPv4 mask itself in subnet-mask field.
            type: int
          mask_length4:
            description:
              - IPv4 network mask length.
            type: int
          mask_length6:
            description:
              - IPv6 network mask length.
            type: int
          subnet_mask:
            description:
              - IPv4 network mask.
            type: str
          color:
            description:
              - Color of the object. Should be one of existing colors.
            type: str
            choices:
              - 'aquamarine'
              - 'black'
              - 'blue'
              - 'crete blue'
              - 'burlywood'
              - 'cyan'
              - 'dark green'
              - 'khaki'
              - 'orchid'
              - 'dark orange'
              - 'dark sea green'
              - 'pink'
              - 'turquoise'
              - 'dark blue'
              - 'firebrick'
              - 'brown'
              - 'forest green'
              - 'gold'
              - 'dark gold'
              - 'gray'
              - 'dark gray'
              - 'light green'
              - 'lemon chiffon'
              - 'coral'
              - 'sea green'
              - 'sky blue'
              - 'magenta'
              - 'purple'
              - 'slate blue'
              - 'violet red'
              - 'navy blue'
              - 'olive'
              - 'orange'
              - 'red'
              - 'sienna'
              - 'yellow'
          comments:
            description:
              - Comments string.
            type: str
          details_level:
            description:
              - The level of detail for some of the fields in the response can vary from showing
                only the UID value of the object to a fully detailed representation of the object.
            type: str
            choices:
              - 'uid'
              - 'standard'
              - 'full'
          ignore_warnings:
            description:
              - Apply changes ignoring warnings.
            type: bool
          ignore_errors:
            description:
              - Apply changes ignoring errors. You won't be able to publish such a changes.
              - If ignore-warnings flag was omitted - warnings will also be ignored.
            type: bool
      nat_settings:
        description: NAT settings.
        type: dict
        suboptions:
          auto_rule:
            description:
              - Whether to add automatic address translation rules.
            type: bool
          ip_address:
            description:
              - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly.
              - This parameter is not required in case "method" parameter is "hide" and "hide-behind" parameter is "gateway".
            type: str
          ipv4_address:
            description:
              - IPv4 address.
            type: str
          ipv6_address:
            description:
              - IPv6 address.
            type: str
          hide_behind:
            description:
              - Hide behind method. This parameter is not required in case "method" parameter is "static".
            type: str
            choices:
              - 'gateway'
              - 'ip-address'
          install_on:
            description:
              - Which gateway should apply the NAT translation.
            type: str
          method:
            description:
              - NAT translation method.
            type: str
            choices:
              - 'hide'
              - 'static'
      tags:
        description: Collection of tag identifiers.
        type: list
        elements: str
      host_servers:
        description: Servers Configuration.
        type: dict
        suboptions:
          dns_server:
            description: Gets True if this server is a DNS Server.
            type: bool
          mail_server:
            description: Gets True if this server is a Mail Server.
            type: bool
          web_server:
            description: Gets True if this server is a Web Server.
            type: bool
          web_server_config:
            description: Web Server configuration.
            type: dict
            suboptions:
              additional_ports:
                description:
                  - Server additional ports.
                type: list
                elements: str
              application_engines:
                description:
                  - Application engines of this web server.
                type: list
                elements: str
              listen_standard_port:
                description:
                  - Whether server listens to standard port.
                type: bool
              operating_system:
                description:
                  - Operating System.
                type: str
                choices:
                  - 'sparc linux'
                  - 'windows'
                  - 'other'
                  - 'x86 linux'
                  - 'sparc solaris'
              protected_by:
                description:
                  - Network object which protects this server identified by the name or UID.
                type: str
      color:
        description: Color of the object. Should be one of existing colors.
        type: str
        choices:
        - aquamarine
        - black
        - blue
        - crete blue
        - burlywood
        - cyan
        - dark green
        - khaki
        - orchid
        - dark orange
        - dark sea green
        - pink
        - turquoise
        - dark blue
        - firebrick
        - brown
        - forest green
        - gold
        - dark gold
        - gray
        - dark gray
        - light green
        - lemon chiffon
        - coral
        - sea green
        - sky blue
        - magenta
        - purple
        - slate blue
        - violet red
        - navy blue
        - olive
        - orange
        - red
        - sienna
        - yellow
      comments:
        description: Comments string.
        type: str
      details_level:
        description: The level of detail for some of the fields in the response can
          vary from showing only the UID value of the object to a fully detailed representation
          of the object.
        type: str
        choices:
        - uid
        - standard
        - full
      groups:
        description: Collection of group identifiers.
        type: list
        elements: str
      ignore_warnings:
        description: Apply changes ignoring warnings.
        type: bool
      ignore_errors:
        description: Apply changes ignoring errors. You won't be able to publish such
          a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
        type: bool
      limit:
        description:
          - The maximal number of returned results.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: int
      offset:
        description:
          - Number of the results to initially skip.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: int
      order:
        description:
          - Sorts results by the given field. By default the results are sorted in the ascending order by name.
            This parameter is relevant only for getting few objects.
          - NOTE, this parameter is a valid parameter only for the GATHERED state, for config states
            like, MERGED, REPLACED, and DELETED state it won't be applicable.
        type: list
        elements: dict
        suboptions:
          ASC:
            description:
              - Sorts results by the given field in ascending order.
            type: str
          DESC:
            description:
              - Sorts results by the given field in descending order.
            type: str
      round_trip:
        description:
          - If set to True, the round trip will filter out the module parameters from the response param,
            which will enable the user to fire the config request using the structured gathered data.
          - NOTE, this parameter makes relevance only with the GATHERED state, as for config states like,
            MERGED, REPLACED, and DELETED state it won't make any config updates,
            as it's not a module config parameter.
        type: bool
      auto_publish_session:
        description:
          - Publish the current session if changes have been performed
            after task completes.
        type: bool
      version:
        description:
          - Version of checkpoint. If not given one, the latest version taken.
        type: str
  state:
    description:
    - The state the configuration should be left in
    - The state I(gathered) will get the module API configuration from the device
      and transform it into structured data in the format as per the module argspec
      and the value is returned in the I(gathered) key within the result.
    type: str
    choices:
    - merged
    - replaced
    - gathered
    - deleted
"""

EXAMPLES = """

# Using MERGED state
# -------------------

- name: Merge MGMT Hosts config
  cp_mgmt_hosts:
    state: merged
    config:
      color: cyan
      ip_address: 192.0.2.1
      name: New Host 1
      auto_publish_session: true
      tags:
        - New Host
      round_trip: true

# RUN output:
# -----------

# mgmt_hosts:
#   after:
#     color: cyan
#     comments: ''
#     groups: []
#     icon: Objects/host
#     interfaces: []
#     ipv4-address: 192.0.2.1
#     name: New Host 1
#     nat_settings: {}
#     tags:
#     - New Host
#   before: {}

# Using REPLACED state
# --------------------

- name: Replace MGMT Host config
  cp_mgmt_hosts:
    state: replaced
    config:
      name: New Host 1
      tags:
        - New Replaced Host
      color: aquamarine
      ip_address: 198.51.110.0
      comments: REPLACED description
      ignore_warnings: true
      ignore_errors: false
      auto_publish_session: true
      round_trip: true

# RUN output:
# -----------

# mgmt_hosts:
#   after:
#     color: aquamarine
#     comments: REPLACED description
#     groups: []
#     icon: Objects/host
#     interfaces: []
#     ipv4-address: 198.51.110.0
#     name: New Host 1
#     nat_settings: {}
#     tags:
#     - New Replaced Host
#   before:
#     color: cyan
#     comments: ''
#     groups: []
#     icon: Objects/host
#     interfaces: []
#     ipv4-address: 192.0.2.1
#     name: New Host 1
#     nat_settings: {}
#     tags:
#     - New Host

# Using GATHERED state
# --------------------

# 1. With Round Trip set to True

- name: Gather MGMT Host config by Name
  cp_mgmt_hosts:
    state: gathered
    config:
      name: New Host 1

# RUN output:
# -----------

# gathered:
#   color: cyan
#   comments: REPLACED description
#   domain: SMC User
#   groups: []
#   icon: Objects/host
#   interfaces: []
#   ipv4-address: 192.0.2.1
#   name: New Host 1
#   nat_settings: {}
#   read-only: false
#   tags:
#   - New Host
#   uid: 63b868bb-d300-47f4-b97a-c465a56fe9c7

# 2. With Round Trip set to False which is the default behaviour

- name: Gather MGMT Host config by Name
  cp_mgmt_hosts:
    state: gathered
    config:
      name: New Host 1

# RUN output:
# -----------

# gathered:
#   color: cyan
#   comments: ''
#   domain:
#     domain-type: domain
#     name: SMC User
#     uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#   groups: []
#   icon: Objects/host
#   interfaces: []
#   ipv4-address: 192.0.2.1
#   meta-info:
#     creation-time:
#       iso-8601: 2022-11-21T08:31+0000
#       posix: 1669019480328
#     creator: admin
#     last-modifier: admin
#     last-modify-time:
#       iso-8601: 2022-11-21T08:31+0000
#       posix: 1669019480328
#     lock: unlocked
#     validation-state: ok
#   name: New Host 1
#   nat_settings: {}
#   read-only: false
#   tags:
#   - domain:
#       domain-type: domain
#       name: SMC User
#       uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#     name: New Host
#     type: tag
#     uid: 94d53896-3cee-4e1f-a83b-3abac80bf512
#   type: host
#   uid: 8f23a44b-d9d2-4242-8a9e-2a4cbb6723ff

# 3. Gather ALL threat-layer config with DESC order filter

- name: Gather All hosts on the MGMT instance
  cp_mgmt_hosts:
    config:
      details_level: full
    state: gathered

# RUN output:
# -----------

# gathered:
#   - domain:
#       domain-type: domain
#       name: SMC User
#       uid: 41e821a0-3720-11e3-aa6e-0800200c9fde
#     ipv4-address: 192.0.2.1
#     name: New Host 1
#     type: host
#     uid: 8f23a44b-d9d2-4242-8a9e-2a4cbb6723ff

# Using DELETED state
# -------------------

- name: Delete MGMT Host config by Name
  cp_mgmt_hosts:
    state: deleted
    config:
      name: New Host 1
      round_trip: true

# RUN output:
# -----------

# mgmt_hosts:
#   after: {}
#   before:
#     color: cyan
#     comments: REPLACED description
#     groups: []
#     icon: Objects/host
#     interfaces: []
#     ipv4-address: 192.0.2.1
#     name: New Host 1
#     nat_settings: {}
#     tags:
#     - New Host
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when state is I(merged), I(replaced), I(deleted)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when state is I(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: custom_issue_definitions_v1
short_description: Resource module for Custom Issue Definitions V1
description:
- Manage operations create, update and delete of the resource Custom Issue Definitions V1.
- >
   Create a new custom issue definition using the provided input request data. The unique identifier for this issue
   definition is id. Please note that the issue names cannot be duplicated. The definition is based on the syslog.
   For detailed information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
- >
   Deletes an existing custom issue definition based on the Id. Only the Global profile issue has the access to
   delete the issue definition, so no profile id is required. For detailed information about the usage of the API,
   please refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-
   api-specs/blob/main/Assurance/CE_Cat_Center_Org-AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
- >
   Updates an existing custom issue definition based on the provided Id. For detailed information about the usage of
   the API, please refer to the Open API specification document - https //github.com/cisco-en-
   programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceUserDefinedIssueAPIs-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Description.
    type: str
  headers:
    description: Additional headers.
    type: dict
  id:
    description: Id path parameter. The custom issue definition Identifier.
    type: str
  isEnabled:
    description: Is Enabled.
    type: bool
  isNotificationEnabled:
    description: Is Notification Enabled.
    type: bool
  name:
    description: Name.
    type: str
  priority:
    description: Priority.
    type: str
  rules:
    description: Custom Issue Definitions's rules.
    elements: dict
    suboptions:
      durationInMinutes:
        description: Duration In Minutes.
        type: int
      facility:
        description: Facility.
        type: str
      mnemonic:
        description: Mnemonic.
        type: str
      occurrences:
        description: Occurrences.
        type: int
      pattern:
        description: Pattern.
        type: str
      severity:
        description: Severity.
        type: int
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues CreatesANewUserDefinedIssueDefinitionsV1
  description: Complete reference of the CreatesANewUserDefinedIssueDefinitionsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!creates-a-new-user-defined-issue-definitions
- name: Cisco DNA Center documentation for Issues DeletesAnExistingCustomIssueDefinitionV1
  description: Complete reference of the DeletesAnExistingCustomIssueDefinitionV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-an-existing-custom-issue-definition
- name: Cisco DNA Center documentation for Issues UpdatesAnExistingCustomIssueDefinitionBasedOnTheProvidedIdV1
  description: Complete reference of the UpdatesAnExistingCustomIssueDefinitionBasedOnTheProvidedIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!updates-an-existing-custom-issue-definition-based-on-the-provided-id
notes:
  - SDK Method used are
    issues.Issues.creates_a_new_user_defined_issue_definitions_v1,
    issues.Issues.deletes_an_existing_custom_issue_definition_v1,
    issues.Issues.updates_an_existing_custom_issue_definition_based_on_the_provided_id_v1,

  - Paths used are
    post /dna/intent/api/v1/customIssueDefinitions,
    delete /dna/intent/api/v1/customIssueDefinitions/{id},
    put /dna/intent/api/v1/customIssueDefinitions/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.custom_issue_definitions_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    headers: '{{my_headers | from_json}}'
    isEnabled: true
    isNotificationEnabled: true
    name: string
    priority: string
    rules:
    - durationInMinutes: 0
      facility: string
      mnemonic: string
      occurrences: 0
      pattern: string
      severity: 0

- name: Update by id
  cisco.dnac.custom_issue_definitions_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    description: string
    headers: '{{my_headers | from_json}}'
    id: string
    isEnabled: true
    isNotificationEnabled: true
    name: string
    priority: string
    rules:
    - durationInMinutes: 0
      facility: string
      mnemonic: string
      occurrences: 0
      pattern: string
      severity: 0

- name: Delete by id
  cisco.dnac.custom_issue_definitions_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "description": "string",
        "profileId": "string",
        "triggerId": "string",
        "rules": [
          {
            "type": "string",
            "severity": 0,
            "facility": "string",
            "mnemonic": "string",
            "pattern": "string",
            "occurrences": 0,
            "durationInMinutes": 0
          }
        ],
        "isEnabled": true,
        "priority": "string",
        "isDeletable": true,
        "isNotificationEnabled": true,
        "createdTime": 0,
        "lastUpdatedTime": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_eip
version_added: 5.0.0
short_description: manages EC2 elastic IP (EIP) addresses.
description:
  - This module can allocate or release an EIP.
  - This module can associate/disassociate an EIP with instances or network interfaces.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  device_id:
    description:
      - The id of the device for the EIP.
      - Can be an EC2 Instance id or Elastic Network Interface (ENI) id.
      - When specifying an ENI id, O(in_vpc) must be V(true).
      - The C(instance_id) alias was removed in release 6.0.0.
    required: false
    type: str
  public_ip:
    description:
      - The IP address of a previously allocated EIP.
      - When O(state=present) and device is specified, the EIP is associated with the device.
      - When O(state=absent) and device is specified, the EIP is disassociated from the device.
    aliases: [ ip ]
    type: str
  state:
    description:
      - When O(state=present), allocate an EIP or associate an existing EIP with a device.
      - When O(state=absent), disassociate the EIP from the device and optionally release it.
    choices: ['present', 'absent']
    default: present
    type: str
  in_vpc:
    description:
      - Allocate an EIP inside a VPC or not.
      - Required if specifying an ENI with O(device_id).
    default: false
    type: bool
  reuse_existing_ip_allowed:
    description:
      - Reuse an EIP that is not associated to a device (when available), instead of allocating a new one.
    default: false
    type: bool
  release_on_disassociation:
    description:
      - Whether or not to automatically release the EIP when it is disassociated.
    default: false
    type: bool
  private_ip_address:
    description:
      - The primary or secondary private IP address to associate with the Elastic IP address.
    type: str
  allow_reassociation:
    description:
      -  Specify this option to allow an Elastic IP address that is already associated with another
         network interface or instance to be re-associated with the specified instance or interface.
    default: false
    type: bool
  tag_name:
    description:
      - When O(reuse_existing_ip_allowed=true), supplement with this option to only reuse
        an Elastic IP if it is tagged with O(tag_name).
    type: str
  tag_value:
    description:
      - Supplements O(tag_name) but also checks that the value of the tag provided in O(tag_name) matches O(tag_value).
    type: str
  public_ipv4_pool:
    description:
      - Allocates the new Elastic IP from the provided public IPv4 pool (BYOIP)
        only applies to newly allocated Elastic IPs, isn't validated when O(reuse_existing_ip_allowed=true).
    type: str
  domain_name:
    description: The domain name to attach to the IP address.
    required: false
    type: str
    version_added: 8.3.0
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3

author:
  - "Rick Mendes (@rickmendes) <rmendes@illumina.com>"
notes:
  - There may be a delay between the time the EIP is assigned and when
    the cloud instance is reachable via the new address. Use wait_for and
    pause to delay further playbook execution until the instance is reachable,
    if necessary.
  - This module returns multiple changed statuses on disassociation or release.
    It returns an overall status based on any changes occurring. It also returns
    individual changed statuses for disassociation and release.
  - Support for O(tags) and O(purge_tags) was added in release 2.1.0.
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: associate an elastic IP with an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003
    ip: 93.184.216.119

- name: associate an elastic IP with a device
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    ip: 93.184.216.119

- name: associate an elastic IP with a device and allow reassociation
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    public_ip: 93.184.216.119
    allow_reassociation: true

- name: disassociate an elastic IP from an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003
    ip: 93.184.216.119
    state: absent

- name: disassociate an elastic IP with a device
  amazon.aws.ec2_eip:
    device_id: eni-c8ad70f3
    ip: 93.184.216.119
    state: absent

- name: allocate a new elastic IP and associate it with an instance
  amazon.aws.ec2_eip:
    device_id: i-1212f003

- name: allocate a new elastic IP without associating it to anything
  amazon.aws.ec2_eip:
    state: present
  register: eip

- name: output the IP
  ansible.builtin.debug:
    msg: "Allocated IP is {{ eip.public_ip }}"

- name: provision new instances with ec2
  amazon.aws.ec2:
    keypair: mykey
    instance_type: c1.medium
    image: ami-40603AD1
    wait: true
    group: webserver
    count: 3
  register: ec2

- name: associate new elastic IPs with each of the instances
  amazon.aws.ec2_eip:
    device_id: "{{ item }}"
  loop: "{{ ec2.instance_ids }}"

- name: allocate a new elastic IP inside a VPC in us-west-2
  amazon.aws.ec2_eip:
    region: us-west-2
    in_vpc: true
  register: eip

- name: output the IP
  ansible.builtin.debug:
    msg: "Allocated IP inside a VPC is {{ eip.public_ip }}"

- name: allocate eip - reuse unallocated ips (if found) with FREE tag
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: FREE

- name: allocate eip - reuse unallocated ips if tag reserved is nope
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: reserved
    tag_value: nope

- name: allocate new eip - from servers given ipv4 pool
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02

- name: allocate eip - from a given pool (if no free addresses where dev-servers tag is dynamic)
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: dev-servers
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02

- name: allocate eip from pool - check if tag reserved_for exists and value is our hostname
  amazon.aws.ec2_eip:
    region: us-east-1
    in_vpc: true
    reuse_existing_ip_allowed: true
    tag_name: reserved_for
    tag_value: "{{ inventory_hostname }}"
    public_ipv4_pool: ipv4pool-ec2-0588c9b75a25d1a02

- name: create new IP and modify it's reverse DNS record
  amazon.aws.ec2_eip:
    state: present
    domain_name: test-domain.xyz

- name: Modify reverse DNS record of an existing EIP
  amazon.aws.ec2_eip:
    public_ip: 44.224.84.105
    domain_name: test-domain.xyz
    state: present

- name: Remove reverse DNS record of an existing EIP
  amazon.aws.ec2_eip:
    public_ip: 44.224.84.105
    domain_name: ""
    state: present
"""

RETURN = r"""
allocation_id:
  description: Allocation id of the elastic ip.
  returned: on success
  type: str
  sample: eipalloc-51aa3a6c
public_ip:
  description: An elastic ip address.
  returned: on success
  type: str
  sample: 52.88.159.209
update_reverse_dns_record_result:
  description: Information about result of update reverse dns record operation.
  returned: When O(domain_name) is specified.
  type: dict
  contains:
    address:
      description: Information about the Elastic IP address.
      returned: always
      type: dict
      contains:
        allocation_id:
          description: The allocation ID.
          returned: always
          type: str
          sample: "eipalloc-00a11aa111aaa1a11"
        ptr_record:
          description: The pointer (PTR) record for the IP address.
          returned: always
          type: str
          sample: "ec2-11-22-33-44.us-east-2.compute.amazonaws.com."
        ptr_record_update:
          description: The updated PTR record for the IP address.
          returned: always
          type: dict
          contains:
            status:
              description: The status of the PTR record update.
              returned: always
              type: str
              sample: "PENDING"
            value:
              description: The value for the PTR record update.
              returned: always
              type: str
              sample: "example.com"
        public_ip:
          description: The public IP address.
          returned: always
          type: str
          sample: "11.22.33.44"
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import allocate_address as allocate_ip_address
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import associate_address
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_addresses
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_instances
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_network_interfaces
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import disassociate_address
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import release_address
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def find_address(
    ec2, public_ip: Optional[str], device_id: Optional[str], is_instance
) -> Optional[Dict[str, Union[str, List[Dict[str, str]]]]]:
    """Find an existing Elastic IP address"""
    filters = None

    if not public_ip and not device_id:
        return None

    if public_ip:
        params = {"PublicIps": [public_ip]}
    else:
        if is_instance:
            filters = [{"Name": "instance-id", "Values": [device_id]}]
        else:
            filters = [{"Name": "network-interface-id", "Values": [device_id]}]
        params = {"Filters": filters}

    result = None
    addresses = describe_addresses(ec2, **params)
    if addresses:
        if len(addresses) > 1:
            raise AnsibleEC2Error(f"Found more than one address using args {params} Addresses found: {addresses}")
        result = addresses[0]
    return result


def address_is_associated_with_device(
    ec2,
    address: Optional[Dict[str, str]],
    device_id: str,
    is_instance: bool = True,
) -> Optional[str]:
    """Check if the elastic IP is currently associated with the device and return the association Id"""
    public_ip = None if not address else address.get("PublicIp")
    result = find_address(ec2, public_ip, device_id, is_instance)
    association_id = None
    if result:
        instance_id = result.get("InstanceId") if is_instance else result.get("NetworkInterfaceId")
        if instance_id == device_id:
            association_id = result.get("AssociationId")
    return association_id


def allocate_address(
    client,
    check_mode: bool,
    search_tags: Optional[Dict[str, str]],
    domain: Optional[str],
    reuse_existing_ip_allowed: bool,
    tags: Optional[Dict[str, str]],
    public_ipv4_pool: Optional[bool] = None,
) -> Tuple[Dict[str, str], bool]:
    """Allocate a new elastic IP address (when needed) and return it"""
    if not domain:
        domain = "standard"

    if reuse_existing_ip_allowed:
        filters = []
        filters.append({"Name": "domain", "Values": [domain]})

        if search_tags is not None:
            filters += ansible_dict_to_boto3_filter_list(search_tags)

        all_addresses = describe_addresses(client, Filters=filters)

        if domain == "vpc":
            unassociated_addresses = [a for a in all_addresses if not a.get("AssociationId", None)]
        else:
            unassociated_addresses = [a for a in all_addresses if not a["InstanceId"]]
        if unassociated_addresses:
            return unassociated_addresses[0], False

    params = {"Domain": domain}
    if public_ipv4_pool:
        params.update({"PublicIpv4Pool": public_ipv4_pool})
    if tags:
        params["TagSpecifications"] = boto3_tag_specifications(tags, types="elastic-ip")
    address = None
    if not check_mode:
        address = allocate_ip_address(client, **params)
    return address, True


def find_device(client, device_id: str, is_instance: bool) -> Optional[Dict[str, Any]]:
    """Attempt to find the EC2 instance and return it"""

    result = None
    if is_instance:
        reservations = describe_instances(client, InstanceIds=[device_id])
        if len(reservations) == 1:
            instances = reservations[0]["Instances"]
            if len(instances) == 1:
                result = instances[0]
    else:
        interfaces = describe_network_interfaces(client, NetworkInterfaceIds=[device_id])
        if len(interfaces) == 1:
            result = interfaces[0]
    return result


def generate_tag_dict(module: AnsibleAWSModule) -> Optional[Dict[str, str]]:
    """Generates a dictionary to be passed as a filter to Amazon"""
    tag_name = module.params.get("tag_name")
    tag_value = module.params.get("tag_value")
    result = None

    if not tag_name:
        return result

    if not tag_value:
        if tag_name.startswith("tag:"):
            tag_name = tag_name.strip("tag:")
        result = {"tag-key": tag_name}
    else:
        if not tag_name.startswith("tag:"):
            tag_name = "tag:" + tag_name
        result = {tag_name: tag_value}

    return result


def check_is_instance(module: AnsibleAWSModule) -> bool:
    device_id = module.params.get("device_id")
    in_vpc = module.params.get("in_vpc")
    if not device_id:
        return False
    if device_id.startswith("i-"):
        return True

    if device_id.startswith("eni-") and not in_vpc:
        raise module.fail_json("If you are specifying an ENI, in_vpc must be true")
    return False


def ensure_absent(
    client: Any, module: AnsibleAWSModule, address: Optional[Dict[str, Any]], is_instance: bool
) -> Dict[str, bool]:
    disassociated = False
    released = False

    device_id = module.params.get("device_id")
    release_on_disassociation = module.params.get("release_on_disassociation")

    if address:
        if device_id:
            # disassociating address from instance
            association_id = address_is_associated_with_device(client, address, device_id, is_instance)
            if association_id:
                disassociated = True
                if not module.check_mode:
                    disassociated = disassociate_address(client, association_id=association_id)

        if not device_id or (disassociated and release_on_disassociation):
            # Release or Release on disassociation
            released = True
            if not module.check_mode:
                released = release_address(client, allocation_id=address["AllocationId"])
    return {"changed": disassociated or released, "disassociated": disassociated, "released": released}


def ensure_present(
    client, module: AnsibleAWSModule, address: Optional[Dict[str, Any]], is_instance: bool
) -> Dict[str, Any]:
    device_id = module.params.get("device_id")
    private_ip_address = module.params.get("private_ip_address")
    in_vpc = module.params.get("in_vpc")
    domain = "vpc" if in_vpc else None
    reuse_existing_ip_allowed = module.params.get("reuse_existing_ip_allowed")
    allow_reassociation = module.params.get("allow_reassociation")
    public_ipv4_pool = module.params.get("public_ipv4_pool")
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")
    domain_name = module.params.get("domain_name")

    # Tags for *searching* for an EIP.
    search_tags = generate_tag_dict(module)
    result = {}
    changed = False

    # Allocate address
    if not address:
        address, changed = allocate_address(
            client, module.check_mode, search_tags, domain, reuse_existing_ip_allowed, tags, public_ipv4_pool
        )

    if domain_name is not None:
        changed, update_reverse_dns_record_result = update_reverse_dns_record_of_eip(
            client, module, address, domain_name
        )
        result.update({"update_reverse_dns_record_result": update_reverse_dns_record_result})

    # Associate address to instance
    if device_id:
        # Find instance
        instance = find_device(client, device_id, is_instance)
        # Allocate an IP for instance since no public_ip was provided
        if is_instance and reuse_existing_ip_allowed:
            if instance["VpcId"] and len(instance["VpcId"]) > 0 and domain is None:
                raise AnsibleEC2Error(
                    "You must set 'in_vpc' to true to associate an instance with an existing ip in a vpc"
                )

        # check if the address is already associated to the device
        association_id = address_is_associated_with_device(client, address, device_id, is_instance)
        if not association_id:
            changed = True
            if not module.check_mode:
                # Associate address object (provided or allocated) with instance
                if is_instance:
                    params = {"InstanceId": device_id, "AllowReassociation": allow_reassociation}
                    if address.get("Domain") == "vpc":
                        params["AllocationId"] = address.get("AllocationId")
                    else:
                        params["PublicIp"] = address.get("PublicIp")
                else:
                    params = {
                        "NetworkInterfaceId": device_id,
                        "AllocationId": address.get("AllocationId"),
                        "AllowReassociation": allow_reassociation,
                    }

                if private_ip_address:
                    params["PrivateIpAddress"] = private_ip_address
                associate_address(client, **params)

    # Ensure tags
    if address:
        changed |= ensure_ec2_tags(
            client, module, address["AllocationId"], resource_type="elastic-ip", tags=tags, purge_tags=purge_tags
        )
        result.update({"public_ip": address["PublicIp"], "allocation_id": address["AllocationId"]})

    result["changed"] = changed
    return result


def update_reverse_dns_record_of_eip(client, module: AnsibleAWSModule, address, domain_name):
    if module.check_mode:
        return True, {}

    current_ptr_record_domain = client.describe_addresses_attribute(
        AllocationIds=[address["AllocationId"]], Attribute="domain-name"
    )

    if (
        current_ptr_record_domain["Addresses"]
        and current_ptr_record_domain["Addresses"][0]["PtrRecord"] == domain_name + "."
    ):
        return False, {"ptr_record": domain_name + "."}

    if len(domain_name) == 0:
        try:
            update_reverse_dns_record_result = client.reset_address_attribute(
                AllocationId=address["AllocationId"], Attribute="domain-name"
            )
            changed = True
        except AnsibleEC2Error as e:
            module.fail_json_aws_error(e)

        if "ResponseMetadata" in update_reverse_dns_record_result:
            del update_reverse_dns_record_result["ResponseMetadata"]
    else:
        try:
            update_reverse_dns_record_result = client.modify_address_attribute(
                AllocationId=address["AllocationId"], DomainName=domain_name
            )
            changed = True
        except AnsibleEC2Error as e:
            module.fail_json_aws_error(e)

        if "ResponseMetadata" in update_reverse_dns_record_result:
            del update_reverse_dns_record_result["ResponseMetadata"]

    return changed, camel_dict_to_snake_dict(update_reverse_dns_record_result)


def main():
    argument_spec = dict(
        device_id=dict(required=False),
        domain_name=dict(required=False, type="str"),
        public_ip=dict(required=False, aliases=["ip"]),
        state=dict(required=False, default="present", choices=["present", "absent"]),
        in_vpc=dict(required=False, type="bool", default=False),
        reuse_existing_ip_allowed=dict(required=False, type="bool", default=False),
        release_on_disassociation=dict(required=False, type="bool", default=False),
        allow_reassociation=dict(type="bool", default=False),
        private_ip_address=dict(),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(required=False, type="bool", default=True),
        tag_name=dict(),
        tag_value=dict(),
        public_ipv4_pool=dict(),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_by={
            "private_ip_address": ["device_id"],
            "tag_value": ["tag_name"],
        },
    )

    ec2 = module.client("ec2")

    device_id = module.params.get("device_id")
    public_ip = module.params.get("public_ip")
    state = module.params.get("state")

    is_instance = check_is_instance(module)

    try:
        # Find existing address
        address = find_address(ec2, public_ip, device_id, is_instance)
        if state == "present":
            result = ensure_present(ec2, module, address, is_instance)
        else:
            result = ensure_absent(ec2, module, address, is_instance)

    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from ansible_collections.amazon.aws.plugins.module_utils import botocore as botocore_utils


class ModuleDocFragment:
    # Modules and Plugins can (currently) use the same fragment
    def __init__(self):
        # Minimum requirements for the collection
        requirements = f"""
options: {{}}
requirements:
  - python >= 3.6
  - boto3 >= {botocore_utils.MINIMUM_BOTO3_VERSION}
  - botocore >= {botocore_utils.MINIMUM_BOTOCORE_VERSION}
"""

        self.DOCUMENTATION = requirements
        self.MODULES = requirements
        self.PLUGINS = requirements



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr_acl_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.acl_interfaces import (
    Acl_interfacesTemplate,
)


class Acl_interfaces(ResourceModule):
    """
    The iosxr_acl_interfaces class
    """

    def __init__(self, module):
        super(Acl_interfaces, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="acl_interfaces",
            tmplt=Acl_interfacesTemplate(),
        )

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.gen_config()
            self.run_commands()
        return self.result

    def gen_config(self):
        """Select the appropriate function based on the state provided

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        # convert list of dicts to dicts of dicts
        wantd = {entry["name"]: entry for entry in self.want}
        haved = {entry["name"]: entry for entry in self.have}

        # turn all lists of dicts into dicts prior to merge
        for entry in wantd, haved:
            self._list_to_dict(entry)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        begin = len(self.commands)
        self._compare_lists(want=want, have=have)
        if len(self.commands) != begin:
            self.commands.insert(
                begin,
                self._tmplt.render(want or have, "interface", False),
            )

    def _compare_lists(self, want, have):
        wdict = want.get("access_groups", {})
        hdict = have.get("access_groups", {})

        for afi in ("ipv4", "ipv6"):
            wacls = wdict.pop(afi, {}).pop("acls", {})
            hacls = hdict.pop(afi, {}).pop("acls", {})

            for key, entry in wacls.items():
                if entry != hacls.pop(key, {}):
                    entry["afi"] = afi
                    self.addcmd(entry, "access_groups", False)
            # remove remaining items in have for replaced
            for entry in hacls.values():
                entry["afi"] = afi
                self.addcmd(entry, "access_groups", True)

    def _list_to_dict(self, entry):
        for item in entry.values():
            for ag in item.get("access_groups", []):
                ag["acls"] = {subentry["direction"]: subentry for subentry in ag.get("acls", [])}
            item["access_groups"] = {
                subentry["afi"]: subentry for subentry in item.get("access_groups", [])
            }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_profiling_rules_count_v1_info
short_description: Information module for Endpoint Analytics Profiling Rules Count V1
description:
- Get all Endpoint Analytics Profiling Rules Count V1.
- >
   This API fetches the count of profiling rules based on the filter values provided in the query parameters. The
   filter parameters are same as that of 'GET /profiling-rules' API, excluding the pagination and sort parameters.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  ruleType:
    description:
    - RuleType query parameter. Use comma-separated list of rule types to filter the data. Defaults to 'Custom Rule'.
    type: str
  includeDeleted:
    description:
    - IncludeDeleted query parameter. Flag to indicate whether deleted rules should be part of the records fetched.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.get_count_of_profiling_rules_v1,

  - Paths used are
    get /dna/intent/api/v1/endpoint-analytics/profiling-rules/count,

"""

EXAMPLES = r"""
- name: Get all Endpoint Analytics Profiling Rules Count V1
  cisco.dnac.endpoint_analytics_profiling_rules_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    ruleType: string
    includeDeleted: True
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "count": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pan_ha_update
short_description: Resource module for Pan Ha Update
description:
- Manage operation update of the resource Pan Ha Update.
- To deploy the auto-failover feature, you must have at least three nodes, where.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  failedAttempts:
    description: Failover occurs if the primary PAN is down for the specified number
      of failure polls. Count (2 - 60).<br> The default value is 5.
    type: int
  isEnabled:
    description: IsEnabled flag.
    type: bool
  pollingInterval:
    description: Administration nodes are checked after each interval. Seconds (30 -
      300) <br> The default value is 120.
    type: int
  primaryHealthCheckNode:
    description: Pan Ha Update's primaryHealthCheckNode.
    suboptions:
      hostname:
        description: Pan Ha Update's hostname.
        type: str
    type: dict
  secondaryHealthCheckNode:
    description: Pan Ha Update's secondaryHealthCheckNode.
    suboptions:
      hostname:
        description: Pan Ha Update's hostname.
        type: str
    type: dict
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for PAN HA
  description: Complete reference of the PAN HA API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deployment-openapi
notes:
  - SDK Method used are
    pan_ha.PanHa.update_pan_ha,

  - Paths used are
    put /api/v1/deployment/pan-ha,

"""

EXAMPLES = r"""
- name: Update all
  cisco.ise.pan_ha_update:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    failedAttempts: 0
    isEnabled: true
    pollingInterval: 0
    primaryHealthCheckNode:
      hostname: string
    secondaryHealthCheckNode:
      hostname: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2023 Patrick Uiterwijk <@puiterwijk>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_accesstoken_info

version_added: "1.19.0"

short_description: Get Azure API access token

description:
    - Get an access token for Azure APIs.

options:
    scopes:
        description:
            - The scopes to request.
        type: list
        elements: str
        required: True
    claims:
        description:
            - Additional claims required in the token.
        type: list
        elements: str
    token_tenant_id:
        description:
            - Tenant to include in the token request.
        type: str
    enable_cae:
        description:
            - Whether to enable Continuous Access Evaluation (CAE) for the requested token.
        default: false
        type: bool

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Patrick Uiterwijk (@puiterwijk)
'''

EXAMPLES = '''
- name: Get access token for Microsoft Graph
  azure.azcollection.azure_rm_accesstoken_info:
    scopes:
      - https://graph.microsoft.com/.default
'''

RETURN = '''
access_token:
    description:
        - API access token.
    returned: success
    type: str
    sample: eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwiaWF0IjoxNTE2MjM5MDIyfQ.L8i6g3PfcHlioHCCPURC9pmXT7gdJpx3kOoyAfNUwCc
expires_on:
    description:
        - Timestamp the token expires on.
    returned: success
    type: int
    sample: 1699337824
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMAccessToken(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            scopes=dict(type='list', elements='str', required=True),
            claims=dict(type='list', elements='str'),
            token_tenant_id=dict(type='str'),
            enable_cae=dict(type='bool', default=False),
        )

        self.scopes = None
        self.claims = None
        self.token_tenant_id = None
        self.enable_cae = False

        self.results = dict(changed=False)

        super(AzureRMAccessToken, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=False,
                                                 is_ad_resource=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        claims = None
        if self.claims is not None:
            claims = ' '.join(self.claims)

        cred = self.azure_auth.azure_credential_track2
        token = cred.get_token(
            *self.scopes,
            claims=claims,
            tenant_id=self.token_tenant_id,
            enable_cae=self.enable_cae,
        )

        self.results['access_token'] = token.token
        self.results['expires_on'] = token.expires_on
        return self.results


def main():
    AzureRMAccessToken()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_domain
short_description: Manage physical, virtual, bridged, routed or FC domain profiles (phys:DomP, vmm:DomP, l2ext:DomP, l3ext:DomP, and fc:DomP)
description:
- Manage physical, virtual, bridged, routed or FC domain profiles on Cisco ACI fabrics.
options:
  domain:
    description:
    - Name of the physical, virtual, bridged routed or FC domain profile.
    type: str
    aliases: [ domain_name, domain_profile, name ]
  domain_type:
    description:
    - The type of domain profile.
    - 'C(fc): The FC domain profile is a policy pertaining to single FC Management domain'
    - 'C(l2dom): The external bridged domain profile is a policy for managing L2 bridged infrastructure bridged outside the fabric.'
    - 'C(l3dom): The external routed domain profile is a policy for managing L3 routed infrastructure outside the fabric.'
    - 'C(phys): The physical domain profile stores the physical resources and encap resources that should be used for EPGs associated with this domain.'
    - 'C(vmm): The VMM domain profile is a policy for grouping VM controllers with similar networking policy requirements.'
    type: str
    required: true
    choices: [ fc, l2dom, l3dom, phys, vmm ]
    aliases: [ type ]
  dscp:
    description:
    - The target Differentiated Service (DSCP) value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target ]
  encap_mode:
    description:
    - The layer 2 encapsulation protocol to use with the virtual switch.
    type: str
    choices: [ unknown, vlan, vxlan ]
  add_infra_pg:
    description:
    - Configure port groups for infra VLAN (e.g. Virtual APIC).
    type: bool
    aliases: [ infra_pg ]
  tag_collection:
    description:
    - Enables Cisco APIC to collect VMs that have been assigned tags in VMware vCenter for microsegmentation.
    type: bool
  multicast_address:
    description:
    - The multicast IP address to use for the virtual switch.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  vm_provider:
    description:
    - The VM platform for VMM Domains.
    - Support for Kubernetes was added in ACI v3.0.
    - Support for CloudFoundry, OpenShift and Red Hat was added in ACI v3.1.
    type: str
    choices: [ cloudfoundry, kubernetes, microsoft, openshift, openstack, redhat, vmware ]
  vswitch:
    description:
    - The virtual switch to use for vmm domains.
    - The APIC defaults to C(default) when unset during creation.
    type: str
    choices: [ avs, default, dvs, unknown ]
  access_mode:
    description:
    - Access mode for vmm domains
    - This parameter cannot be changed after a domain is created
    type: str
    choices: [ read-only, read-write ]
  enable_vm_folder:
    description:
    - Enable VM folder data retrieval
    type: bool
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- module: cisco.aci.aci_aep_to_domain
- module: cisco.aci.aci_domain_to_encap_pool
- module: cisco.aci.aci_domain_to_vlan_pool
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(phys:DomP),
               B(vmm:DomP), B(l2ext:DomP), B(l3ext:DomP) and B(fc:DomP)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dag Wieers (@dagwieers)
"""

EXAMPLES = r"""
- name: Add a new physical domain
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: phys_dom
    domain_type: phys
    state: present

- name: Remove a physical domain
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: phys_dom
    domain_type: phys
    state: absent

- name: Add a new VMM domain
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: hyperv_dom
    domain_type: vmm
    vm_provider: microsoft
    state: present
  delegate_to: localhost

- name: Remove a VMM domain
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: hyperv_dom
    domain_type: vmm
    vm_provider: microsoft
    state: absent
  delegate_to: localhost

- name: Query a specific physical domain
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain: phys_dom
    domain_type: phys
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all domains
  cisco.aci.aci_domain:
    host: apic
    username: admin
    password: SomeSecretPassword
    domain_type: phys
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec

VM_PROVIDER_MAPPING = dict(
    cloudfoundry="CloudFoundry",
    kubernetes="Kubernetes",
    microsoft="Microsoft",
    openshift="OpenShift",
    openstack="OpenStack",
    redhat="Redhat",
    vmware="VMware",
)

VSWITCH_MAPPING = dict(
    avs="n1kv",
    default="default",
    dvs="default",
    unknown="unknown",
)

BOOL_TO_ACI_MAPPING = {True: "yes", False: "no", None: None}


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        domain_type=dict(type="str", required=True, choices=["fc", "l2dom", "l3dom", "phys", "vmm"], aliases=["type"]),
        domain=dict(type="str", aliases=["domain_name", "domain_profile", "name"]),  # Not required for querying all objects
        dscp=dict(
            type="str",
            choices=[
                "AF11",
                "AF12",
                "AF13",
                "AF21",
                "AF22",
                "AF23",
                "AF31",
                "AF32",
                "AF33",
                "AF41",
                "AF42",
                "AF43",
                "CS0",
                "CS1",
                "CS2",
                "CS3",
                "CS4",
                "CS5",
                "CS6",
                "CS7",
                "EF",
                "VA",
                "unspecified",
            ],
            aliases=["target"],
        ),
        encap_mode=dict(type="str", choices=["unknown", "vlan", "vxlan"]),
        add_infra_pg=dict(type="bool", aliases=["infra_pg"]),
        tag_collection=dict(type="bool"),
        multicast_address=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        vm_provider=dict(type="str", choices=["cloudfoundry", "kubernetes", "microsoft", "openshift", "openstack", "redhat", "vmware"]),
        vswitch=dict(type="str", choices=["avs", "default", "dvs", "unknown"]),
        name_alias=dict(type="str"),
        access_mode=dict(type="str", choices=["read-write", "read-only"]),
        enable_vm_folder=dict(type="bool"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["domain_type", "vmm", ["vm_provider"]],
            ["state", "absent", ["domain", "domain_type"]],
            ["state", "present", ["domain", "domain_type"]],
        ],
    )

    dscp = module.params.get("dscp")
    domain = module.params.get("domain")
    domain_type = module.params.get("domain_type")
    encap_mode = module.params.get("encap_mode")
    add_infra_pg = BOOL_TO_ACI_MAPPING[module.params.get("add_infra_pg")]
    tag_collection = BOOL_TO_ACI_MAPPING[module.params.get("tag_collection")]
    multicast_address = module.params.get("multicast_address")
    vm_provider = module.params.get("vm_provider")
    vswitch = module.params.get("vswitch")
    if vswitch is not None:
        vswitch = VSWITCH_MAPPING.get(vswitch)
    access_mode = module.params.get("access_mode")
    enable_vm_folder = BOOL_TO_ACI_MAPPING[module.params.get("enable_vm_folder")]
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    if domain_type != "vmm":
        if vm_provider is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'vm_provider'".format(domain_type))
        if encap_mode is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'encap_mode'".format(domain_type))
        if multicast_address is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'multicast_address'".format(domain_type))
        if vswitch is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'vswitch'".format(domain_type))
        if access_mode is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'access_mode'".format(domain_type))
        if enable_vm_folder is not None:
            module.fail_json(msg="Domain type '{0}' cannot have parameter 'enable_vm_folder'".format(domain_type))

    if dscp is not None and domain_type not in ["l2dom", "l3dom"]:
        module.fail_json(msg="DSCP values can only be assigned to 'l2ext and 'l3ext' domains")

    # Compile the full domain for URL building
    if domain_type == "fc":
        domain_class = "fcDomP"
        domain_mo = "uni/fc-{0}".format(domain)
        domain_rn = "fc-{0}".format(domain)
    elif domain_type == "l2dom":
        domain_class = "l2extDomP"
        domain_mo = "uni/l2dom-{0}".format(domain)
        domain_rn = "l2dom-{0}".format(domain)
    elif domain_type == "l3dom":
        domain_class = "l3extDomP"
        domain_mo = "uni/l3dom-{0}".format(domain)
        domain_rn = "l3dom-{0}".format(domain)
    elif domain_type == "phys":
        domain_class = "physDomP"
        domain_mo = "uni/phys-{0}".format(domain)
        domain_rn = "phys-{0}".format(domain)
    elif domain_type == "vmm":
        domain_class = "vmmDomP"
        domain_mo = "uni/vmmp-{0}/dom-{1}".format(VM_PROVIDER_MAPPING.get(vm_provider), domain)
        domain_rn = "vmmp-{0}/dom-{1}".format(VM_PROVIDER_MAPPING.get(vm_provider), domain)

    # Ensure that querying all objects works when only domain_type is provided
    if domain is None:
        domain_mo = None

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class=domain_class,
            aci_rn=domain_rn,
            module_object=domain_mo,
            target_filter={"name": domain},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=domain_class,
            class_config=dict(
                encapMode=encap_mode,
                mcastAddr=multicast_address,
                configInfraPg=add_infra_pg,
                enableTag=tag_collection,
                mode=vswitch,
                name=domain,
                targetDscp=dscp,
                nameAlias=name_alias,
                accessMode=access_mode,
                enableVmFolder=enable_vm_folder,
            ),
        )

        aci.get_diff(aci_class=domain_class)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: projects_details_info
short_description: Information module for Projects Details Info
description:
- This module represents an alias of the module projects_details_v2_info
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id query parameter. Id of project to be searched.
    type: str
  name:
    description:
    - Name query parameter. Name of project to be searched.
    type: str
  offset:
    description:
    - Offset query parameter. Index of first result.
    type: int
  limit:
    description:
    - Limit query parameter. Limits number of results.
    type: int
  sortOrder:
    description:
    - SortOrder query parameter. Sort Order Ascending (asc) or Descending (dsc).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates GetProjectsDetailsV2
  description: Complete reference of the GetProjectsDetailsV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-projects-details-v-2
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.get_projects_details_v2,

  - Paths used are
    get /dna/intent/api/v2/template-programmer/project,
  - It should be noted that this module is an alias of projects_details_v2_info

"""

EXAMPLES = r"""
- name: Get all Projects Details Info
  cisco.dnac.projects_details_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    name: string
    offset: 0
    limit: 0
    sortOrder: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "createTime": 0,
      "description": "string",
      "id": "string",
      "isDeletable": true,
      "lastUpdateTime": 0,
      "name": "string",
      "tags": [
        {
          "id": "string",
          "name": "string"
        }
      ],
      "templates": {}
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: images_site_wise_product_names_v1
short_description: Resource module for Images Site Wise Product Names V1
description:
- Manage operations create, update and delete of the resource Images Site Wise Product Names V1.
- >
   Assign network device product name and sites for the given image identifier. Refer `/dna/intent/api/v1/images` API
   for obtaining imageId.
- This API unassigns the network device product name from all the sites for the given software image.
- >
   Update the list of sites for the network device product name assigned to the software image. Refer to
   `/dna/intent/api/v1/images` and `/dna/intent/api/v1/images/{imageId}/siteWiseProductNames` GET APIs for obtaining
   `imageId` and `productNameOrdinal` respectively.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  imageId:
    description: ImageId path parameter. Software image identifier. Refer `/dna/intent/api/v1/images`
      API for obtaining `imageId`.
    type: str
  productNameOrdinal:
    description: Product name ordinal is unique value for each network device product.
    type: float
  siteIds:
    description: Sites where this image needs to be assigned. Ref https //developer.cisco.com/docs/dna-center/#!sites.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) AssignNetworkDeviceProductNameToTheGivenSoftwareImageV1
  description: Complete reference of the AssignNetworkDeviceProductNameToTheGivenSoftwareImageV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!assign-network-device-product-name-to-the-given-software-image
- name: Cisco DNA Center documentation for Software Image Management (SWIM) UnassignNetworkDeviceProductNameFromTheGivenSoftwareImageV1
  description: Complete reference of the UnassignNetworkDeviceProductNameFromTheGivenSoftwareImageV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!unassign-network-device-product-name-from-the-given-software-image
- name: Cisco DNA Center documentation for Software Image Management (SWIM) UpdateTheListOfSitesForTheNetworkDeviceProductNameAssignedToTheSoftwareImageV1
  description: Complete reference of the UpdateTheListOfSitesForTheNetworkDeviceProductNameAssignedToTheSoftwareImageV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-the-list-of-sites-for-the-network-device-product-name-assigned-to-the-software-image
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.assign_network_device_product_name_to_the_given_software_image_v1,
    software_image_management_swim.SoftwareImageManagementSwim.unassign_network_device_product_name_from_the_given_software_image_v1,
    software_image_management_swim.SoftwareImageManagementSwim.update_the_list_of_sites_for_the_network_device_product_name_assigned_to_the_software_image_v1,

  - Paths used are
    post /dna/intent/api/v1/images/{imageId}/siteWiseProductNames,
    delete /dna/intent/api/v1/images/{imageId}/siteWiseProductNames/{productNameOrdinal},
    put /dna/intent/api/v1/images/{imageId}/siteWiseProductNames/{productNameOrdinal},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.images_site_wise_product_names_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    imageId: string
    productNameOrdinal: 0
    siteIds:
    - string

- name: Delete by name
  cisco.dnac.images_site_wise_product_names_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    imageId: string
    productNameOrdinal: 0

- name: Update by name
  cisco.dnac.images_site_wise_product_names_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    imageId: string
    productNameOrdinal: 0
    siteIds:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: endpoint_analytics_endpoints_anc_policy_update
short_description: Resource module for Endpoint Analytics Endpoints Anc Policy Update
description:
- This module represents an alias of the module endpoint_analytics_endpoints_anc_policy_update_v1
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  ancPolicy:
    description: ANC policy name.
    type: str
  epId:
    description: EpId path parameter. Unique identifier for the endpoint.
    type: str
  granularAncPolicy:
    description: Endpoint Analytics Endpoints Anc Policy Update's granularAncPolicy.
    elements: dict
    suboptions:
      name:
        description: Name of the granular ANC policy.
        type: str
      nasIpAddress:
        description: IP address of the network device where endpoint is attached.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for AI Endpoint Analytics ApplyANCPolicyV1
  description: Complete reference of the ApplyANCPolicyV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!apply-anc-policy
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.apply_anc_policy_v1,

  - Paths used are
    put /dna/intent/api/v1/endpoint-analytics/endpoints/{epId}/anc-policy,
  - It should be noted that this module is an alias of endpoint_analytics_endpoints_anc_policy_update_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.endpoint_analytics_endpoints_anc_policy_update:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    ancPolicy: string
    epId: string
    granularAncPolicy:
    - name: string
      nasIpAddress: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: site_delete
short_description: Resource module for Site Delete
description:
- This module represents an alias of the module site_delete_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  siteId:
    description: SiteId path parameter. Site id to which site details to be deleted.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites DeleteSiteV1
  description: Complete reference of the DeleteSiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-site
notes:
  - SDK Method used are
    sites.Sites.delete_site_v1,

  - Paths used are
    delete /dna/intent/api/v1/site/{siteId},
  - It should be noted that this module is an alias of site_delete_v1

"""

EXAMPLES = r"""
- name: Delete by id
  cisco.dnac.site_delete:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    siteId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Evpn_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Evpn_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Evpn_globalTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "default_gateway.advertise",
            "getval": re.compile(
                r"""
                \sdefault-gateway\sadvertise
                $""",
                re.VERBOSE,
            ),
            "setval": "default-gateway advertise",
            "result": {"default_gateway": {"advertise": True}},
        },
        {
            "name": "flooding_suppression.address_resolution.disable",
            "getval": re.compile(
                r"""
                \sflooding-suppression\saddress-resolution\sdisable
                $""",
                re.VERBOSE,
            ),
            "setval": "flooding-suppression address-resolution disable",
            "result": {
                "flooding_suppression": {
                    "address_resolution": {
                        "disable": True,
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "ip.local_learning.disable",
            "getval": re.compile(
                r"""
                \sip\slocal-learning\sdisable
                $""",
                re.VERBOSE,
            ),
            "setval": "ip local-learning disable",
            "result": {
                "ip": {
                    "local_learning": {
                        "disable": True,
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "replication_type",
            "getval": re.compile(
                r"""
                \sreplication-type\s(?P<replication_type>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "replication-type {{replication_type}}",
            "result": {"replication_type": "{{replication_type}}"},
            "shared": True,
        },
        {
            "name": "route_target.auto.vni",
            "getval": re.compile(
                r"""
                \sroute-target\sauto\svni
                $""",
                re.VERBOSE,
            ),
            "setval": "route-target auto vni",
            "result": {"route_target": {"auto": {"vni": True}}},
            "shared": True,
        },
        {
            "name": "router_id",
            "getval": re.compile(
                r"""
                \srouter-id\s(?P<router_id>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "router-id {{ router_id }}",
            "result": {"router_id": "{{router_id}}"},
            "shared": True,
        },
    ]
    # fmt: on
